unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, FIBSQLMonitor, SoIBSQLMonitor, SoChkBox,
  SoFrmStorage, SohoBtns, CheckLst, SoCtrls;

type
  TMainForm = class(TForm)
    sohoLabel1: TsohoLabel;
    AppL: TsohoComboBox;
    TraceM: TMemo;
    sohoLabel2: TsohoLabel;
    GroupBox1: TGroupBox;
    TraceOptions: TCheckListBox;
    EnableMonC: TsohoCheckBox;
    MainIni: TsohoFormStorage;
    CloseB: TsohoBitBtn;
    sohoLabel3: TsohoLabel;
    ShowTimeC: TsohoCheckBox;
    ShowTypeC: TsohoCheckBox;
    sohoBitBtn1: TsohoBitBtn;
    ClearB: TsohoBitBtn;
    procedure MonitorNewApplication(Sender: TObject;
      const ApplicationName: String; var AddToTraceList: Boolean);
    procedure CloseBClick(Sender: TObject);
    procedure EnableMonCChange(Sender: TObject);
    procedure TraceOptionsClickCheck(Sender: TObject);
    procedure MonitorTrace(Sender: TObject; const ApplicationName,
      SQLObjectName: String; TraceOperation: TsohoTraceOption;
      const SQLText: String);
    procedure ClearBClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    Monitor : TsohoIBSQLMonitor;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation
uses SoStrUtils;

{$R *.DFM}

procedure TMainForm.MonitorNewApplication(Sender: TObject;
  const ApplicationName: String; var AddToTraceList: Boolean);
begin
  AppL.Items.Add(ApplicationName);
  if AppL.Items.Count = 1 then begin
    AppL.ItemIndex := 0;
    Monitor.TraceApplication := ApplicationName;
  end;
end;

procedure TMainForm.CloseBClick(Sender: TObject);
begin
  Monitor.Free;
  Close;
end;

procedure TMainForm.EnableMonCChange(Sender: TObject);
begin
  if EnableMonC.Checked then begin
    Monitor := TsohoIBSQLMonitor.Create(Self);
    Monitor.OnTrace := MonitorTrace;
    Monitor.OnNewApplication := MonitorNewApplication;
  end
  else begin
    Monitor.Free;
  end;
end;

const
   TrOptions : array [0..8] of TsohoTraceOption = (
      toConnect,
      toDisconnect,
      toTransactionStart,
      toCommit,
      toCommitRetaining,
      toRollback,
      toSQLPrepare,
      toSQLExecute,
      toSQLFetch
   );
procedure TMainForm.TraceOptionsClickCheck(Sender: TObject);
var Index: LongInt;
begin
  Monitor.TraceOptions := [];
  for Index := 0 to pred(TraceOptions.Items.Count) do
    if TraceOptions.Checked[Index] then
      Monitor.TraceOptions := Monitor.TraceOptions + [TrOptions[Index]];
end;

const
  OptionsNames : array [TsohoTraceOption] of string = (
      'Connect',
      'Disconnect',
      'Transaction Start',
      'Commit',
      'Commit Retaining',
      'Rollback',
      'Prepare Query',
      'Execute Query',
      'Fetch record',
      ''
  );
procedure TMainForm.MonitorTrace(Sender: TObject; const ApplicationName,
  SQLObjectName: String; TraceOperation: TsohoTraceOption;
  const SQLText: String);
var Title: string;
begin
  Title := '';
  if ShowTimeC.Checked then Title := TimeToStr(Time) + ': ';
  if ShowTypeC.Checked then Title := Title + OptionsNames[TraceOperation];
  if Title <> '' then TraceM.Lines.Add(Title);
  if SQLText <> '' then TraceM.Lines.Text := TraceM.Lines.Text + SQLText;
  if (Title <> '') or (SQLText <> '') then
    TraceM.Lines.Add(Chars('_', 40));
  if TraceM.Lines.Count > 1 then
    while (TraceM.Lines.Count > 1) and (TraceM.Lines[1] = '') do
      TraceM.Lines.Delete(1); 
end;

procedure TMainForm.ClearBClick(Sender: TObject);
begin
  TraceM.Lines.Clear;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var Index: LongInt;
begin
  for Index := 0 to pred(TraceOptions.Items.Count) do
    TraceOptions.Checked[Index] := true;
  TraceM.Lines.Clear;
end;

end.
