unit Global;

interface
uses Classes, SoClass, SoIBClasses, SysUtils, Controls, SoMemTree;

type

   PNewsNode = ^TNewsNode;
   TNewsNode = record
     Id: Integer;
     LinkId: Integer; // Id Server or Id NewsServerlist
     Name: string;
     // For servers
     Reply: string;
     Login: string;
     Password: string;
   end;

   TProject = class(TComponent)
   private
     FId: Integer;
     FName: string;
     FStart: TDate;
     FUrl, FEmail: string;
     FSign: string;
     FRegMessage, FNews: string;
     FSMTP, FPOP3, FUserName, FPassword: string;
     FNewsServers: TStringList;
     FDatabase: TsohoIBDatabase;
     FTransaction: TsohoIBTransaction;
     FQuery: TsohoIBSQL;
     FNewsTree: TsohoMemoryTree;
     procedure SetNewsServers(const Value: TStringList);
     procedure SetDatabase(const Value: TsohoIBDatabase);
     procedure SetTransaction(const Value: TsohoIBTransaction);
     procedure SetId(const Value: Integer);
     procedure NewsNodeDelete(Sender : TObject; Item : pointer);
   public
     constructor Create(aOwner: TComponent); override;
     destructor Destroy; override;
     procedure LoadSettings;
     procedure SaveSettings;

     procedure AddNewsServer(Id, ServerId: Integer;
       const aName, Reply, Login, Password: string);
     procedure DeleteNewsServer(Index: Integer);
     procedure AddNewsGroup(ServerIndex: Integer; const aName: string);
     procedure DeleteNewsGroup(ServerIndex: Integer; Index: Integer);
     function ServerIndexById(Id: Integer): Integer;

     property Id: Integer read FId write SetId;
     property Name: string read FName write FName;
     property Start: TDate read FStart write FStart;
     property Url: string read FUrl write FUrl;
     property Email: string read FEmail write FEmail;
     property Signature: string read FSign write FSign;
     property RegistrationMessage: string read FRegMessage write FRegMessage;
     property NewsMessage: string read FNews write FNews;
     property SMTPServer: string read FSMTP write FSMTP;
     property POP3Server: string read FPOP3 write FPOP3;
     property UserName: string read FUserName write FUserName;
     property Password: string read FPassword write FPassword;
     property NewsServers: TStringList read FNewsServers write SetNewsServers;
     property News: TsohoMemoryTree read FNewsTree;
     property Database: TsohoIBDatabase read FDatabase write SetDatabase;
     property Transaction: TsohoIBTransaction read FTransaction write SetTransaction;
   end;

var ProjectOptions: TProject;

implementation
uses SoDate, SoStrUtils, SoUnit;

{ TProject }

procedure TProject.AddNewsGroup(ServerIndex: Integer; const aName: string);
var Parent: TsohoMTreeItem;
    Tmp: PNewsNode;
begin
  Parent := FNewsTree.Root.List.Nodes[ServerIndex];
  New(Tmp);
  Tmp.Name := aName;
  Tmp.Id := -1;
  FNewsTree.AddItem(Parent, Tmp);
end;

procedure TProject.AddNewsServer(Id, ServerId: Integer; const aName,
  Reply, Login, Password: string);
var Tmp: PNewsNode;
begin
  New(Tmp);
  Tmp.Name := aName;
  Tmp.Id := Id;
  Tmp.LinkId := ServerId;
  Tmp.Reply := Reply;
  Tmp.Login := Login;
  Tmp.Password := Password;
  FNewsTree.AddItem(FNewsTree.Root, Tmp);
end;

constructor TProject.Create(aOwner: TComponent);
begin
  inherited;
  FNewsServers := TStringList.Create;
  FQuery := TsohoIBSQL.Create(Self);
  FId := -1;
  FNewsTree := TsohoMemoryTree.Create;
  FNewsTree.OnItemDelete := NewsNodeDelete;
end;

procedure TProject.DeleteNewsGroup(ServerIndex, Index: Integer);
var Parent: TsohoMTreeItem;
begin
  Parent := FNewsTree.Root.List.Nodes[ServerIndex];
  Parent.List.Delete(Index);
end;

procedure TProject.DeleteNewsServer(Index: Integer);
begin
  FNewsTree.DeleteItem(FNewsTree.Root.List.Nodes[Index]);
end;

destructor TProject.Destroy;
begin
  FQuery.Close;
  FQuery.Free;
  FNewsServers.Free;
  FNewsTree.Free;
  inherited;
end;

procedure TProject.LoadSettings;
var ServerId, ServerIndex: Integer;
begin
  with FQuery do begin
    SQL.Clear;
    SQL.Add('select * from "Projects" where "Id" = ' + IntToStr(Id));
    try
      if not Transaction.InTransaction then Transaction.StartTransaction;
      ExecSQL;
      if not Eof then begin
        FName := FieldByName['Name'].AsString;
        FStart := FieldByName['Start Date'].AsDate;
        FUrl := FieldByName['URL'].AsString;
        FEmail := FieldByName['E-Mail'].AsString;
        FSign := FieldByName['Signature'].AsString;
        FRegMessage := FieldByName['Registration Message'].AsString;
        FNews := FieldByName['Newsletter Template'].AsString;
        FSMTP := FieldByName['SMTP Server'].AsString;
        FPOP3 := FieldByName['POP3 Server'].AsString;
        FUserName := FieldByName['Mail Login'].AsString;
        FPassword := FieldByName['Mail Password'].AsString;
      end;
      Close;
    except
      Transaction.Rollback;
      exit;
    end;

    FNewsTree.Clear;
    
    SQL.Clear;
    SQL.Add('SELECT');
    SQL.Add('  "News Servers List"."Id",');
    SQL.Add('  "News Servers List"."Id Project",');
    SQL.Add('  "News Servers List"."Id News Server",');
    SQL.Add('  "News Servers"."Address",');
    SQL.Add('  "News Servers"."Reply E-Mail",');
    SQL.Add('  "News Servers"."Login",');
    SQL.Add('  "News Servers"."Password"');
    SQL.Add('FROM "News Servers"');
    SQL.Add('INNER JOIN "News Servers List" ON ("News Servers"."Id" = "News Servers List"."Id News Server")');
    SQL.Add('where "Id Project" = ' + IntToStr(Id));
    try
      if not Transaction.InTransaction then Transaction.StartTransaction;
      ExecSQL;
      while not Eof do begin
        AddNewsServer(
          FieldByName['Id'].AsInteger,
          FieldByName['Id News Server'].AsInteger,
          FieldByName['Address'].AsString,
          FieldByName['Reply E-Mail'].AsString,
          FieldByName['Login'].AsString,
          FieldByName['Password'].AsString
          );
        Next;
      end;
      Close;
    except
      Transaction.Rollback;
      exit;
    end;

    SQL.Clear;
    SQL.Add('SELECT');
    SQL.Add('  "Newsgroups"."Name",');
    SQL.Add('  "News Servers List"."Id News Server",');
    SQL.Add('  "News Servers List"."Id Project"');
    SQL.Add('FROM "News Servers List"');
    SQL.Add('   INNER JOIN "Newsgroups" ON ("News Servers List"."Id" = "Newsgroups"."Id Newsserver List")');
    SQL.Add('where "Id Project" = ' + IntToStr(Id));
    try
      if not Transaction.InTransaction then Transaction.StartTransaction;
      ExecSQL;
      ServerId := -1;
      ServerIndex := -1;
      while not Eof do begin
        if ServerId <> FieldByName['Id News Server'].AsInteger then
          ServerIndex := ServerIndexById(FieldByName['Id News Server'].AsInteger);
        ServerId := FieldByName['Id News Server'].AsInteger;

        AddNewsGroup(
           ServerIndex,
           FieldByName['Name'].AsString);
        Next;
      end;
      Close;
    except
      Transaction.Rollback;
      exit;
    end;
  end;
end;

procedure TProject.NewsNodeDelete(Sender: TObject; Item: pointer);
begin
  FreeMem(TsohoMTreeItem(Item).Data);
end;

procedure TProject.SaveSettings;
var I, J: Integer;
begin
  with FQuery, SQL do begin
    Clear;
    Add('update "Projects" set');
    Add('  "Name" = ' + SingleQuotes(FName) + ',');
    Add('  "Start Date" = ' + DateToSQLDate(FStart) + ',');
    Add('  "URL" = ' + SingleQuotes(FUrl) + ',');
    Add('  "E-Mail" = ' + SingleQuotes(FEmail) + ',');
    Add('  "Signature" = ' + '?Sign,');
    Add('  "Registration Message" = ?Reg,');
    Add('  "Newsletter Template" = ?News,');
    Add('  "SMTP Server" = ' + SingleQuotes(FSMTP) + ',');
    Add('  "POP3 Server" = ' + SingleQuotes(FPOP3) + ',');
    Add('  "Mail Login" = ' + SingleQuotes(FUserName) + ',');
    Add('  "Mail Password" = ' + SingleQuotes(FPassword));
    Add('  where "Id" = ' + IntToStr(Id));
    try
      if not Transaction.InTransaction then Transaction.StartTransaction;
      Prepare;
      Params[0].AsString := FSign;
      Params[1].AsString := FRegMessage;
      Params[2].AsString := FNews;
      ExecSQL;
      Close;
    except
      Transaction.Rollback;
      exit;
    end;

    //     news-
    SQL.Clear;
    SQL.Add('delete from "News Servers List" where "Id Project" = ' + IntToStr(Id));
    try
      if not Transaction.InTransaction then Transaction.StartTransaction;
      ExecSQL;
      Close;
    except
      Transaction.Rollback;
      exit;
    end;

    Clear;
    Add('insert into "News Servers List" ("Id", "Id News Server", "Id Project")');
    Add('values (?Id, ?IdNews, ?IdProj)');
    try
      if not Transaction.InTransaction then Transaction.StartTransaction;
      Prepare;
      for I := 0 to pred(FNewsTree.Root.List.Count) do begin
        PNewsNode(FNewsTree.Root.List.Nodes[I].Data).Id :=
           GetNewId('News Servers List GEN_Id');
        Params[0].AsInteger := PNewsNode(FNewsTree.Root.List.Nodes[I].Data).Id;
        Params[1].AsInteger := PNewsNode(FNewsTree.Root.List.Nodes[I].Data).LinkId;
        Params[2].AsInteger := FId;
        ExecSQL;
      end;
      // Transaction.Commit;
    except
      Transaction.Rollback;
      exit;
    end;

    Clear;
    Add('insert into "Newsgroups" ("Id Newsserver List", "Name")');
    Add('values (?IdNews, ?Name)');
    try
      if not Transaction.InTransaction then Transaction.StartTransaction;
      Prepare;
      for I := 0 to pred(FNewsTree.Root.List.Count) do begin
        Params[0].AsInteger := PNewsNode(FNewsTree.Root.List.Nodes[I].Data).Id;
        for J := 0 to pred(FNewsTree.Root.List.Nodes[I].List.Count) do begin
          Params[1].AsString := PNewsNode(
            FNewsTree.Root.List.Nodes[I].List.Nodes[J].Data).Name;
          ExecSQL;
        end;
      end;
      Transaction.Commit;
    except
      Transaction.Rollback;
      exit;
    end;

  end;
end;

function TProject.ServerIndexById(Id: Integer): Integer;
var Index: Integer;
begin
  Result := -1;
  for Index := 0 to pred(FNewsTree.Count) do
    if PNewsNode(FNewsTree.Root.List.Nodes[Index].Data).LinkId = Id then begin
      Result := Index;
      exit;
    end;
end;

procedure TProject.SetDatabase(const Value: TsohoIBDatabase);
begin
  FDatabase := Value;
  FQuery.Database := FDatabase;
end;

procedure TProject.SetId(const Value: Integer);
begin
  if FId <> -1 then SaveSettings;
  FId := Value;
  if FId = -1 then begin
    Name := '';
    Start := Date;
    Url := '';
    Email := '';
    Signature := '';
    RegistrationMessage := '';
    NewsMessage := '';
    SMTPServer := '';
    POP3Server := '';
    UserName := '';
    Password := '';
    NewsServers.Clear;
  end
  else LoadSettings;
end;

procedure TProject.SetNewsServers(const Value: TStringList);
begin
  FNewsServers.Assign(Value);
end;

procedure TProject.SetTransaction(const Value: TsohoIBTransaction);
begin
  FTransaction := Value;
  FQuery.Transaction := FTransaction;
end;

end.
