unit NewsLetterF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, OkCancelHelpF, FIBDatabase, pFIBDatabase,
  SoIBClasses, Db, FIBDataSet, pFIBDataSet, SoSplit, Grids, DBGrids,
  SoDBGrid, SoCtmRgs, SoIBRgst, Menus, NMNNTP, Psock, NMsmtp, SoCtrls,
  SoCtmFld, SoIBFolder, SoFrmStorage, FIBQuery, SoCtmDBGridAddon,
  SoIBDBGridAddon;

type
  TNewsLetterForm = class(TForm)
    Panel1: TPanel;
    OkCancelHelpFrame1: TOkCancelHelpFrame;
    Panel2: TPanel;
    Grid: TsohoDBGrid;
    sohoSplitter1: TsohoSplitter;
    ClientsDS: TsohoIBDataSet;
    DataSource1: TDataSource;
    CLientsTr: TsohoIBTransaction;
    Reg: TsohoIBSubRegister;
    ClientsFilter: TPopupMenu;
    ViewClientsM: TMenuItem;
    ViewBetaTestersM: TMenuItem;
    SMTP: TNMSMTP;
    NNTP: TNMNNTP;
    Panel3: TPanel;
    TextM: TMemo;
    Panel4: TPanel;
    sohoLabel1: TsohoLabel;
    SubjE: TsohoEdit;
    Folder: TsohoIBFolder;
    Ini: TsohoFormStorage;
    IBSQL: TsohoIBSQL;
    GridAddon: TsohoIBDBGridAddon;
    procedure ViewClientsMClick(Sender: TObject);
    procedure ClientsDSFilterRecord(DataSet: TDataSet;
      var Accept: Boolean);
    procedure IniSavePlacement(Sender: TObject);
    procedure IniRestorePlacement(Sender: TObject);
    procedure SMTPFailure(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    procedure SendNewsLetter;
    procedure SendNewsByEmail;
    procedure SendNewsToNewsServers;

    procedure BeforeAddon(Sender: TObject);
    procedure AfterAddon(Sender: TObject);
  public
    { Public declarations }
  end;

var
  NewsLetterForm: TNewsLetterForm;

procedure SendNewsLetter;

implementation
uses Main, Global, SoIBRtn, SoDBRtn, SoUnit, SoVCLUtils, SoAnimate;

{$R *.DFM}

procedure SendNewsLetter;
begin
  try
    NewsLetterForm := TNewsLetterForm.Create(Application);
    NewsLetterForm.SendNewsLetter;
  finally
    NewsLetterForm.Free;
  end;
end;

{ TNewsLetterForm }

procedure TNewsLetterForm.SendNewsLetter;
var Progress: TForm;
begin
  TextM.Lines.Text := ProjectOptions.NewsMessage;

  ClientsTr.StartTransaction;

  ClientsDS.SelectSQL.LoadFromFile(Reg.PathToSQL + 'Get clients.sql');
  ClientsDS.Prepare;
  ClientsDS.Params[0].AsInteger := ProjectOptions.Id;
  ClientsDS.Open;

  TextM.Lines.Text := ProjectOptions.NewsMessage;

  if ShowModal = mrOk then begin
    Progress := CreateCopyForm('Sending...', true);
    SendNewsByEmail;
    SendNewsToNewsServers;
    Progress.Free;
  end;
  ClientsTr.Commit;
end;

procedure TNewsLetterForm.ViewClientsMClick(Sender: TObject);
begin
  (Sender as TMenuItem).Checked := not (Sender as TMenuItem).Checked;
  RefreshDataSetFilter(ClientsDS);
end;

procedure TNewsLetterForm.ClientsDSFilterRecord(DataSet: TDataSet;
  var Accept: Boolean);
begin
  Accept := (ViewClientsM.Checked and
             (not DataSet.FieldByName('Is Beta-Tester').AsBoolean)) or
            (ViewBetaTestersM.Checked and
                  DataSet.FieldByName('Is Beta-Tester').AsBoolean);
end;

procedure TNewsLetterForm.SendNewsByEmail;
var NewId: Integer;
begin
  if SMTP.Connected then SMTP.Disconnect;

  SMTP.Host := ProjectOptions.SMTPServer;
  SMTP.UserID := ProjectOptions.UserName;
  try
    SMTP.Connect;
  except
    ErrorMsg('Can not connect to SMTP Server!');
    exit;
  end;

  with SMTP, PostMessage do begin
    FromAddress := ProjectOptions.Email;
    FromName := '';
    ToCarbonCopy.Text := '';
    ToBlindCarbonCopy.Text := '';
    Subject := SubjE.AsString;
    LocalProgram := 'Support Center';
    Date := DateToStr(Now);
    SMTP.PostMessage.Date := DateToStr(Now);
    ReplyTo := ProjectOptions.Email;
    Body.Assign(TextM.Lines);
    Body.Add('');
    Body.Text := Body.Text + ProjectOptions.Signature;
  end;
  with ClientsDS do begin
    First;
    if not Eof then begin
      while not Eof do begin
        SMTP.PostMessage.ToBlindCarbonCopy.Add(FieldByName('E-Mail').AsString);
        Next;
      end;
      try
        SMTP.SendMail;
      except
        ErrorMsg('Error of posting SMTP mail!');
        exit;
      end;
    end;
  end;
  SMTP.Disconnect;

  //     
  with IBSQL, SQL do begin
    Clear;
    Add('insert into "Newsletters" ("Id", "Id Project", "Message Text", "Entry Date", "Subject")');
    Add('values (?Id, ?IdProj, ?NewsText, ?EntryDate, ?Subj)');
    try
      Prepare;
      NewId := GetNewId('News_logging_GEN_Id');
      Params[0].AsInteger := NewId;
      Params[1].AsInteger := ProjectOptions.Id;
      Params[2].AsString := TextM.Lines.Text;
      Params[3].AsDate := Date;
      Params[4].AsString := SubjE.AsString;
      ExecSQL;
    except
      Transaction.RollbackRetaining;
      exit;
    end;

    Clear;
    Add('insert into "Newsletters Logging" ("Id Newsletter", "Id User")');
    Add('values (?IdNews, ?IdUser)');

    try
      Prepare;
      Params[0].AsInteger := NewId;

      ClientsDS.First;
      while not ClientsDS.Eof do begin
        Params[1].AsInteger := ClientsDS.FieldByName('Id User').AsInteger;
        ExecSQL;
        ClientsDS.Next;
      end;
    except
      Transaction.RollbackRetaining;
    end;
  end;
end;

procedure TNewsLetterForm.SendNewsToNewsServers;
var I, J: Integer;
begin
  with ProjectOptions.News do begin
    for I := 0 to pred(Root.List.Count) do begin
      if NNTP.Connected then NNTP.Disconnect;
      NNTP.Host := PNewsNode(Root.List.Nodes[I].Data).Name;
      NNTP.UserId := PNewsNode(Root.List.Nodes[I].Data).Login;
      NNTP.Password := PNewsNode(Root.List.Nodes[I].Data).Password;
      try
        NNTP.Connect;
      except
        if not
          YesNoMsg('Error of connecting to ' + NNTP.Host + '. Continue?') then exit
        else Continue;  
      end;

      for J := 0 to pred(Root.List.Nodes[I].List.Count) do begin
        try
          NNTP.SetGroup(PNewsNode(Root.List.Nodes[I].List.Nodes[J].Data).Name);
          with NNTP.PostRecord do begin
            PrFromAddress := PNewsNode(Root.List.Nodes[I].Data).Reply;
            PrReplyTo := PNewsNode(Root.List.Nodes[I].Data).Reply;
            PrSubject := SubjE.AsString;
            PrAppName := 'Support Center';
            PrNewsGroups := NNTP.SelectedGroup;
            PrTimeDate := DateTimeToStr(Now);
          end;
          NNTP.PostBody.Assign(TextM.Lines);
          NNTP.PostBody.Add('');
          NNTP.PostBody.Text := NNTP.PostBody.Text + ProjectOptions.Signature;
          try
            NNTP.PostArticle;
          except
            ErrorMsg('Error of posting news to ' + NNTP.Host);
          end;
        except
          on E: Exception do ErrorMsg(E.Message);
        end;
      end;

      NNTP.Disconnect;
    end;
  end;
end;

procedure TNewsLetterForm.IniSavePlacement(Sender: TObject);
begin
  Ini.WriteInteger(Ini.IniSection, 'Splitter', Grid.Width);
end;

procedure TNewsLetterForm.IniRestorePlacement(Sender: TObject);
begin
  Grid.Width := Ini.ReadInteger(Ini.IniSection, 'Splitter', Grid.Width);
end;

procedure TNewsLetterForm.SMTPFailure(Sender: TObject);
begin
  ErrorMsg('Error during message sending! Check your SMTP options!');
end;

procedure TNewsLetterForm.FormCreate(Sender: TObject);
begin
  GridAddon.BeforeOperation := BeforeAddon;
  GridAddon.AfterOperation := AfterAddon;
end;

procedure TNewsLetterForm.AfterAddon(Sender: TObject);
begin
  Folder.LoadDBGridColumns;
end;

procedure TNewsLetterForm.BeforeAddon(Sender: TObject);
begin
  Folder.SaveDBGridColumns;
end;

procedure TNewsLetterForm.FormShow(Sender: TObject);
begin
  SubjE.SetFocus;
end;

end.
