{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
unit SoIBClasses;

{$I SOHOIBLIB.INC}

interface

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, DB,
     FIBDatabase, pFIBDatabase, pFIBDataSet, FIBQuery, pFIBStoredProc;
type

  TsohoIBDatabase = class(TpFIBDatabase)
  private
    { Private declarations }
    FOnClose: TNotifyEvent;
    FUserName: string;
    FPassword: string;
    FRole: string;
    FConnectCharSet: string;
  protected
    { Protected declarations }
    procedure GenerateDBParams; virtual;
    procedure InternalClose(Force: Boolean); override;
    procedure SetDataBaseName(const Value: string);
    function GetDatabaseName: string;
  public
    { Public declarations }
    constructor Create(aOwner: TComponent); override;
    procedure Open; override;
    function TableExists(const TableName: string): boolean;
  published
    { Published declarations }
    property DatabaseName: string read GetDatabaseName write SetDatabaseName;
    property UserName: string read FUserName write FUserName;
    property Password: string read FPassword write FPassword;
    property Role: string read FRole write FRole;
    property ConnectCharSet: string read FConnectCharSet write FConnectCharSet;
    property OnClose: TNotifyEvent read FOnCLose write FOnClose;
  end;

  TsohoIBTransactionEvent = procedure (Sender : TObject; Commited: boolean) of object;

  TsohoIBTransaction = class(TpFIBTransaction)
  private
    { Private declarations }
    FOnClose: TsohoIBTransactionEvent;
    FCheckActivity: boolean;
  protected
    { Protected declarations }
    //    TFIBTransaction   
    procedure EndTransaction(Action: TTransactionAction; Force: Boolean); override;
  public
    { Public declarations }
    constructor Create (aOwner: TComponent); override;
    procedure Commit; override; //    TFIBTransaction   
    procedure CommitRetaining; override; //    TFIBTransaction   
    procedure Rollback; override; //    TFIBTransaction   
  published
    { Published declarations }
    property CheckActivity: boolean read FCheckActivity write FCheckActivity default false;
    property OnClose: TsohoIBTransactionEvent read FOnClose write FOnClose;
  end;

  TsohoIBDataSet = class(TpFIBDataSet);

  TsohoIBSQL = class(TFIBQuery)
  public
    procedure ExecSQL;
  end;

  TsohoIBStoredProc = class(TpFIBStoredProc);

function FormatIdentifierValue(Dialect: Integer; const Value: String): string;
function FormatIdentifier(Dialect: Integer; const Value: String): string;

implementation
uses SoStrUtils;

const
   NonQuotedChars = ['A'..'Z', {'a'..'z', }'0'..'9', '_', '$', '%', '#'];

function NeedQuote(const Name: string): boolean;
var Index: integer;
begin
  Result := false;
  if ((Name <> EmptyStr) and (Name[1] = '"')) then exit;
  for Index := 1 to Length(Name) do begin
    Result := not (Name[Index] in NonQuotedChars);
    if Result then Exit;
  end;
end;

function FormatIdentifierValue(Dialect: Integer; const Value: string): string;
begin
  if Dialect = 1 then Result := AnsiUpperCase(Trim(Value))
  else Result := Value;
end;

function FormatIdentifier(Dialect: Integer; const Value: string): string;
begin
  if Dialect = 1 then Result := AnsiUpperCase(Trim(Value))
  else
    if NeedQuote(Value) then Result := '"' + Value + '"'
    else Result := Value
end;

{ TsohoIBDatabase }
constructor TsohoIBDatabase.Create(aOwner: TComponent);
begin
  inherited;
  FUserName := EmptyStr;
  FPassword := EmptyStr;
  FRole := EmptyStr;
  FConnectCharSet := 'WIN1251';
end;

procedure TsohoIBDatabase.InternalClose(Force: Boolean);
begin
  inherited;
  if Assigned(FOnClose) then FOnClose(Self);
end;

procedure TsohoIBDatabase.GenerateDBParams;
var Tst: string;
    Tmp: TStringList;
    Index: LongInt;
begin
  Tmp := TStringList.Create;
  with DBParams do begin
    for Index := 0 to pred(Count) do begin
      Tst := Strings[Index];
      if (Pos('user_name', Tst) <> 0) and (FUserName <> EmptyStr) then Continue;
      if (Pos('password', Tst) <> 0) and (FPassword <> EmptyStr) then Continue;
      if (Pos('sql_role_name', Tst) <> 0) and (FRole <> EmptyStr) then Continue;
      if (Pos('lc_ctype', Tst) <> 0) and (FConnectCharSet <> EmptyStr) then Continue;
      Tmp.Add(Tst);
    end;
    if FUserName <> EmptyStr then Tmp.Add('user_name='+FUserName);
    if FPassword <> EmptyStr then Tmp.Add('password='+FPassword);
    if FRole <> EmptyStr then Tmp.Add('sql_role_name='+FRole);
    if FConnectCharSet <> EmptyStr then Tmp.Add('isc_dpb_lc_ctype='+FConnectCharSet);
    Assign(Tmp);
  end;
  Tmp.Free;
end;

procedure TsohoIBDatabase.SetDataBaseName(const Value: string);
begin
  DBName := Value;
end;

function TsohoIBDatabase.GetDatabaseName: string;
begin
  Result := DBName;
end;

procedure TsohoIBDatabase.Open;
begin
  GenerateDBParams;
  inherited;
end;

function TsohoIBDatabase.TableExists(const TableName: string): boolean;
var TmpTr: TsohoIBTransaction;
    TmpQ: TsohoIBSQL;
begin
  TmpTr := TsohoIBTransaction.Create(Self);
  TmpTr.DefaultDatabase := Self;

  TmpQ := TsohoIBSQL.Create(Self);
  with TmpQ, SQL do begin
    Database := Self;
    Transaction := TmpTr;
    TmpTr.StartTransaction;

    Add('select * from RDB$RELATIONS where (RDB$RELATION_NAME = ' +
      SingleQuotes(TableName) + ') and (RDB$VIEW_SOURCE is null)');
    try
      ExecSQL;
      Result := not Eof;
    except
      Result := false;
    end;
    TmpTr.Commit;
  end;
  TmpQ.Free;
  TmpTr.Free;
end;

{ TsohoIBTransaction }
procedure TsohoIBTransaction.EndTransaction(Action: TTransactionAction; Force: Boolean);
begin
  inherited;
  if Assigned(FOnClose) then FOnClose(Self, Action <> TARollback);
end;

constructor TsohoIBTransaction.Create (aOwner: TComponent);
begin
  inherited;
  FCheckActivity := false;
end;

procedure TsohoIBTransaction.Commit;
begin
  if (not FCheckActivity) and
     (not InTransaction) then exit;
  inherited;
end;

procedure TsohoIBTransaction.CommitRetaining;
begin
  if (not FCheckActivity) and
     (not InTransaction) then exit;
  inherited;
end;

procedure TsohoIBTransaction.Rollback;
begin
  if (not FCheckActivity) and
     (not InTransaction) then exit;
  inherited;
end;

{ TsohoIBSQL }
procedure TsohoIBSQL.ExecSQL;
begin
  ExecQuery;
end;

end.
