{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
{
     TsohoIBFolder.    
    TsohoDBGrid  run-time 
}
unit SoIBFolder;

{$I SOHOIBLIB.INC}

interface

uses SysUtils, WinTypes, WinProcs, Messages, Classes, Controls, SoCtmFld,
     SoIBClasses;

type

  {        TsohoDBGrid 
    run-time .      
       TsohoDBGrid,     
      .    "" TsohoDBGrid ( Grid),
     ini- (FolderName  FolderDir  ).  Auto = true
    ,        DataSet,
      .    DisplayLabel, DisplayFormat 
     , TsohoCustomFolder      
    Calculated-,       .  DataSet
     TsohoDBGrid   ,     
        ,     TableName
  }
  TsohoIBFolder = class(TsohoCustomFolder)
  private
    FDataBase : TsohoIBDatabase;
    FTransaction : TsohoIBTransaction;
  protected
    procedure CreateRealField; override;
    procedure DeleteRealField; override;
    procedure ChangeVisible; override;
    procedure Notification(aComponent : TComponent; aOperation : TOperation);override;
  public
    procedure SetEvents;override;
    constructor Create(aOwner : TComponent);override;
    procedure ReOpen; override;
  published
    {   ""   DataSet.   true
    TsohoIBFolder      
            "Protect".
      ,     false  
      }
    property ProtectView default false;
    property DataBase : TsohoIBDatabase read FDataBase write FDataBase;
    property Transaction : TsohoIBTransaction read FTransaction write FTransaction;
    {   ,      }
    property Grid;
    {      DataSet',  
      Grid  DataSource }
    property Auto;
    {   - }
    property FolderName;
    { ,      }
    property FolderDir;
    {  .   ,    
          }
    property TableName;
    {     }
    property EnabledMenus;
    {      DataSet }
    property OnSaveDataSet;
    {      DataSet }
    property OnLoadDataSet;
    {       Grid }
    property OnChange;
  end;

implementation
uses SoAppUtils, SoDBRtn, SoCrFldF, SoRghtsF, SoIBRgst, SoUnit, SoDBCns,
     SoIBUtil, SoVCLUtils, DB;

procedure TsohoIBFolder.CreateRealField;
var SQLCommand: string;
    TmpQuery: TsohoIBSQL;
begin
  if not CreateNewField(SQLCommand) then exit;
  TmpQuery := nil;
  try
    TmpQuery := TsohoIBSQL.Create(Self);
    with TmpQuery, SQL do begin
      Add('ALTER TABLE ' +
        FormatIdentifier(FDataBase.SQLDialect, TableName) + ' ADD ' + SQLCommand);
      try
        DataSet.Close;
        PushCursor(crHourGlass);
        DataBase := FDataBase;
        Transaction := FTransaction;
        TransactionStart(FTransaction);
        ExecQuery;
        TransactionCommit(FTransaction);
      except
        TransactionRollback(FTransaction);
        ErrorMsg(Format(sohoFolderColumnCreateError, [Self.Name]));
      end;
    end;
  finally
    if TmpQuery <> nil then TmpQuery.Free;
    ReOpen;
    RestoreCursor;
  end;
end;

procedure TsohoIBFolder.ReOpen;
var FDataSet: TDataSet;
begin
  with DataSet do begin
    DisableControls;
    Transaction.Commit;
    Transaction.StartTransaction;
    Close;
    try
      try
        {    ,   
                }
        FDataSet := DataSet;
        RemoveAllFields(FDataSet);
        TsohoIBDataSet(DataSet).SelectSQL.Text :=
          TsohoIBDataSet(DataSet).SelectSQL.Text;
        Open;
      except {   !  DataSet-  ! }
        ErrorMsg(Format(sohoFolderDataSetOpenError, [Self.name]));
      end;
    finally
      EnableControls;
    end;
  end;
end;

procedure TsohoIBFolder.DeleteRealField;
var TmpQuery: TsohoIBSQL;
begin
  if not YesNoMsg(Format(sohoFolderBeforeColumnDelete,
    [Grid.SelectedField.FieldName])) then exit;
  TmpQuery := nil;
  try
    TmpQuery := TsohoIBSQL.Create(Self);
    with TmpQuery, SQL do begin
      Add('ALTER TABLE ' +
        FormatIdentifier(FDataBase.SQLDialect, TableName) + ' DROP ' +
        FormatIdentifier(FDataBase.SQLDialect, Grid.SelectedField.FieldName));
      try
        DataSet.Close;
        PushCursor(crHourGlass);
        DataBase := FDataBase;
        Transaction := FTransaction;
        TransactionStart(FTransaction);
        ExecQuery;
        TransactionCommit(FTransaction);
      except
        TransactionRollback(FTransaction);
        ErrorMsg(Format(sohoFolderColumnDeleteError, [Self.Name]));
      end;
    end;
  finally
    if TmpQuery <> nil then TmpQuery.Free;
    ReOpen;
    RestoreCursor;
  end;
end;

procedure TsohoIBFolder.ChangeVisible;
var FieldRights, aOld, RecordRights : byte;
    IsRightsField : boolean;
begin
  {     ,     }
  if SingleIBRegister = nil then exit;
  IsRightsField := DataSet.FindField('Protect')<>nil;
  aOld := 0;
  if IsRightsField then
    aOld := DataSet.FieldByName('Protect').AsInteger;
  RecordRights := aOld;
  if not SetDataRights(FieldRights, RecordRights, IsRightsField) then exit;
  if (FieldRights in DataReadRights) or
    (FieldRights in DataWriteRights) then
    Grid.SelectedField.Tag := FieldRights
  else InfoMsg(Format(sohoFolderYouCanUse, [SetToStr(DataReadRights + DataWriteRights)]));

  FIni.WriteInteger(Grid.SelectedField.FieldName, 'Tag', Grid.SelectedField.Tag);
  if (aOld<>RecordRights) and (DataSet.FindField('Protect')<>nil) then
    if (RecordRights in DataReadRights) or
       (RecordRights in DataWriteRights) then begin
       with TsohoIBSQL.Create(nil) do begin
         DataBase := FDataBase;
         Transaction := FTransaction;
         TransactionStart(FTransaction);
         SQL.Add('update ' + FormatIdentifier(FDataBase.SQLDialect, TableName) +
                 ' set Protect = '+IntToStr(RecordRights)+
                 ' where ' + FormatIdentifier(FDataBase.SQLDialect, KeyField) +
                 '=' + DataSet.FieldByName(KeyField).AsString);
         try
           TransactionStart(FTransaction);
           ExecQuery;
           TransactionCommit(FTransaction);
         except
           on E: Exception do begin
             TransactionRollback(FTransaction);
             ErrorMsg(Name+': '+E.Message);
           end;
         end;
         Free;
       end;
       RefreshDataSetWithId(DataSet, KeyField);
    end
    else InfoMsg(Format(sohoFolderYouCanUse, [SetToStr(DataReadRights + DataWriteRights)]));
  Changed;
end;

procedure TsohoIBFolder.SetEvents;
begin
  if (FDataBase = nil) and (SingleIBRegister<>nil) then
    FDataBase := SingleIBRegister.DataBase;
  if (FTransaction = nil) and (SingleIBRegister<>nil) then
    FTransaction := SingleIBRegister.Transaction;
  inherited SetEvents;
end;

procedure TsohoIBFolder.Notification(aComponent : TComponent; aOperation : TOperation);
begin
  if (aComponent = FTransaction) and (aOperation = opRemove) then
    FTransaction := nil;
  if (aComponent = FDataBase) and (aOperation = opRemove) then
    FDataBase := nil;
  inherited Notification(aComponent, aOperation);
end;

constructor TsohoIBFolder.Create(aOwner : TComponent);
begin
  inherited Create(aOwner);
  ProtectView := false;
end;

end.
