{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
unit SoIBGuide;

{$I SOHOIBLIB.INC}
interface
uses Classes, SoCtmDBGuide, SoIBClasses, SoIBRec, SoIBFolder,
     SoIBDBGridAddon, DB;

type

  TsohoIBGuideForm = class(TsohoCtmDBGuideForm)
  private
    FRecTr, FTransaction: TsohoIBTransaction;
  public
    constructor CreateGuide(aOwner: TComponent; DBGuide : TsohoCustomDBGuide); override;
    property RecordTransaction: TsohoIBTransaction read FRecTr;
    property Transaction: TsohoIBTransaction read FTransaction;
  end;

  TsohoIBGuide = class(TsohoCustomDBGuide)
  private
    FDatabase: TsohoIBDatabase;
    FTransaction: TsohoIBTransaction;
    FExtTransaction: boolean;
  protected
    function CreateGuideForm: TsohoCtmDBGuideForm; override;
    procedure SetEvents; override;
    procedure SetFolderProperties; override;
    procedure SetRecordProperties; override;
    procedure SetQueryProperties; override;
    procedure SetDBGridAddonProperties; override;
    procedure DoBeforeOpen(DataSet: TDataSet); override;
    function DoDeleteRecord : boolean; override;
    procedure AfterEditRecord; override;
  published
    property Database: TsohoIBDatabase read FDatabase write FDatabase;
    property Transaction: TsohoIBTransaction read FTransaction write FTransaction;
  end;

implementation
uses SoIBRgst, Forms, SoIBRtn;

{ TsohoIBGuideForm }

constructor TsohoIBGuideForm.CreateGuide(aOwner: TComponent; DBGuide : TsohoCustomDBGuide);
begin
  inherited;
  if (DBGuide as TsohoIBGuide).FTransaction = nil
    then begin
      FTransaction := TsohoIBTransaction.Create(Self);
      FRecTr := TsohoIBTransaction.Create(Self);
    end else begin
      FTransaction := (DBGuide as TsohoIBGuide).FTransaction;
      FRecTr       := (DBGuide as TsohoIBGuide).FTransaction;
    end;  { --- if --- }

  FRecordForm := TsohoIBRecordForm.Create(Self);
  TsohoIBRecordForm(FRecordForm).Transaction := FRecTr;

  FFolder := TsohoIBFolder.Create(Self);
  TsohoIBFolder(FFolder).Transaction := FTransaction;

  FGuideDS := TsohoIBDataSet.Create(Self);
  TsohoIBDataSet(FGuideDS).Transaction := FTransaction;

  FDBAddon := TsohoIBDBGridAddon.Create(Self);
end;

{ TsohoIBGuide }

procedure TsohoIBGuide.AfterEditRecord;
begin
  if not FExtTransaction then
    TsohoIBGuideForm(GuideForm).RecordTransaction.Commit;
end;

function TsohoIBGuide.CreateGuideForm: TsohoCtmDBGuideForm;
begin
  Result := TsohoIBGuideForm.CreateGuide(Application, Self);
end;

procedure TsohoIBGuide.DoBeforeOpen(DataSet: TDataSet);
begin
  if FExtTransaction then FTransaction.StartTransaction;
  inherited;
end;

function TsohoIBGuide.DoDeleteRecord: boolean;
var SQLText : string;
begin
  SQLText := 'delete from ' + FormatIdentifier(FDataBase.SQLDialect,
    FolderTable);
  SQLText := SQLText +' where ' +
    FormatIdentifier(FDataBase.SQLDialect, KeyField) + '= ' +
    GuideForm.GuideDS.FieldByName(KeyField).AsString;
  try
    Result := ExecuteSQLText(SQLText, FDataBase,
                             TsohoIBGuideForm(GuideForm).RecordTransaction,
                             NillParams);
    //TsohoIBGuideForm(GuideForm).RecordTransaction.Commit;
  except
    raise;
    //TsohoIBGuideForm(GuideForm).RecordTransaction.Rollback;
  end;  { --- try --- }  
end;

procedure TsohoIBGuide.SetDBGridAddonProperties;
begin
  inherited;
  GuideForm.DBAddon.Grid := GuideForm.GuideGrid; 
end;

procedure TsohoIBGuide.SetEvents;
begin
  if GuideForm = nil then exit;
  if (FDataBase = nil) and (SingleIBRegister <> nil) then
     FDataBase := SingleIBRegister.DataBase;
  TsohoIBGuideForm(GuideForm).GuideSource.DataSet :=
    TsohoIBGuideForm(GuideForm).GuideDS;
  TsohoIBGuideForm(GuideForm).Transaction.DefaultDatabase := FDataBase;
  TsohoIBGuideForm(GuideForm).RecordTransaction.DefaultDatabase := FDataBase;
  if Assigned(FTransaction) then begin
    FExtTransaction := true;
    with TsohoIBGuideForm(GuideForm) do begin
      TsohoIBFolder(Folder).Transaction := FTransaction;
      TsohoIBRecordForm(RecordForm).Transaction := FTransaction;
      TsohoIBRecordForm(RecordForm).AutoCommit := false;
      TsohoIBDataSet(GuideDS).Transaction := FTransaction;
    end;
  end;
  inherited;
end;

procedure TsohoIBGuide.SetFolderProperties;
begin
  inherited;
  with GuideForm do begin
    if Folder.Grid = GuideGrid then exit;
    TsohoIBFolder(Folder).DataBase := FDataBase;
    Folder.Grid := GuideGrid;
    Folder.Activate;
  end
end;

procedure TsohoIBGuide.SetQueryProperties;
var FSQL: string;
begin
  inherited;
  with GuideForm do begin
    if GuideDS.Active then exit;
    FSQL := SQLFile;
    if SingleIBRegister <> nil then
      FSQL := SingleIBRegister.PathToSQL + SQLFile;

    with TsohoIBDataSet(GuideDS), SelectSQL do begin
      DataBase := FDataBase;
      LoadFromFile(FSQL);
    end;
    GuideSource.DataSet := GuideDS;
  end;
end;

procedure TsohoIBGuide.SetRecordProperties;
begin
  inherited;
  with GuideForm do begin
    TsohoIBRecordForm(RecordForm).DataBase := FDataBase;
    TsohoIBRecordForm(RecordForm).Folder := Folder;
  end;
end;

end.
