{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Ported by Serg Vostrikov                }
{          Original code written by A. Malinin          }  
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
unit SoIBLex;

interface
uses Classes;

const
  { Types }
  _EOF        = 0;
  _COMMENT1   = 1;   { * comments *        }
  _KEYWORD    = 2;   { begin, end, ...     }
  _IDENT      = 3;   { _Identifier105      }
  _NUMBER     = 4;   { .5e-10              }
  _LITERAL    = 5;   { ' ''Borland Int.'' '}
  _LITERAL2   = 13;  { " 'Borland' "}
  { 2 characters tokens }
  _NE         = 6;  { <> }
  _GE         = 7;  { >= }
  _LE         = 8;  { <= }
  { 1 character operators }
  _OPSIGN     = 9;  { + - / * < > = }
  _DELIMITER  = 10;  { ( ) [ ] . , ;     }
  _11         = 11; { || }
  _EOL        = 12;
  _UNKNOWN    = 13;

  BufSize     = 200*80; { 200 full lines of comeents }
  BufferSize  = 2048;   { buffer for GetChar }

  IND_SET: LongInt = 0;
  IND_COMMIT: LongInt = 0;
  IND_ROLLBACK: LongInt = 0;
  IND_CREATE: LongInt = 0;
  IND_DROP: LongInt = 0;
  IND_DISCONNECT: LongInt = 0;
  IND_CONNECT: LongInt = 0;
  IND_EXIT: LongInt = 0;
  IND_DOMAIN: LongInt = 0;
  IND_TABLE: LongInt = 0;
  IND_VIEW: LongInt = 0;
  IND_PROCEDURE: LongInt = 0;
  IND_TRIGGER: LongInt = 0;
  IND_GENERATOR: LongInt = 0;
  IND_EXCEPTION: LongInt = 0;
  IND_EXTERNAL: LongInt = 0;
  IND_ALTER: LongInt = 0;
  IND_DECLARE: LongInt = 0;
  IND_ADD: LongInt = 0;
  IND_CONSTRAINT: LongInt = 0;
  IND_AUTODDL: LongInt = 0;
  IND_OFF: LongInt = 0;
  IND_ON: LongInt = 0;
  IND_WORK: LongInt = 0;
  IND_DOCUMENTATION: LongInt = 0;
  IND_COLUMN: LongInt = 0;
  IND_NAMES: LongInt = 0;
  IND_TERM: LongInt = 0;
  IND_DATABASE: LongInt = 0;

type

  LexType     = Integer;
  TokenType   = string;
  TCommentBuf = array[1..BufSize] of Char;
  TBuffer     = array[1..BufferSize] of Char;

  PLexem = ^TLexem;
  TLexem = record
    LexType: Integer;
    TokenValue: TokenType;
    KeywordIndex: LongInt;
  end;

  TIBLexStream = class(TMemoryStream)
  public
    PrevLastPos: LongInt;
    LastPos: LongInt;
  end;

  TLex = class (TObject)
  public
    Stream: TIBLexStream;
    TextLength: LongInt;
    C: char;
    TokenValue: TokenType;     { Lexem }
    SQLDialect: LongInt;
    KeywordIndex: LongInt;
    constructor Create(aStream: TIBLexStream);
    function GetChar: Char;
    function GetLexem: LexType;
  private
    function IsDigit(Ch: Char): Boolean;
    function IsHexDigit(Ch: Char): Boolean;
    function IsAlpha(Ch: Char): Boolean;
    function IsDelimiter(Ch: Char): Boolean;
  end;

var
  KeyWords:    TStringList; { shareable }
  CommentBuf:  TCommentBuf; { ditto     }
  CommentSize: Word;
  Buffer:      TBuffer;

procedure CreateKeywords;  { create KeyWords  }
procedure DestroyKeywords;  { dispose KeyWords }
function LoCaseStr(const S: String): String;

{ ======================================================== }
implementation

function LoCase(Ch: Char): Char;
{ only English letters support }
begin
  if ('A' <= Ch) and (Ch <= 'Z') then
     LoCase := chr( ord(Ch) + ( ord('a') - ord('A') ) )
  else
    LoCase := Ch;
end;

function LoCaseStr(const S: String): String;
var
  i: Byte;
  R: String;
begin
  R := '';
  for i := 1 to length(S) do R := R + LoCase(S[i]);
  LoCaseStr := R;
end;

constructor TLex.Create(aStream: TIBLexStream);
begin
  inherited create;
  Stream := aStream;
  Stream.Position := 0;
  TextLength := Stream.Size;
  SQLDialect := 1;
  { first char must be already read }
  C := GetChar;
  if Keywords = nil then CreateKeywords;
end;

function TLex.GetChar: Char;
var
  Ch: Char;
begin
  Ch := chr(0);
  if Stream.Position < TextLength then Stream.Read(Ch, 1);
  Result := Ch;
end;

function  TLex.GetLexem: LexType;
label
  Start;
var
  C2: Char;
  Finish: Boolean;

  procedure ErrorProc;
  begin
    GetLexem := _UNKNOWN;
    C := GetChar;
  end;

  procedure EndOfText;
  begin
    GetLexem := _EOF;
  end;

  procedure ZeroCommentBuf;
  begin
    CommentSize := 0;
  end;

  procedure WriteComment(Ch: Char);
  begin
    inc(CommentSize);
    CommentBuf[CommentSize] := Ch;
  end;

  procedure Comments1; { / - possible comments or / }
  begin
    C2 := GetChar;
    if C2 = '*' then  { yes - comment } begin
      TokenValue := '';
      { start comment }
      ZeroCommentBuf;
      WriteComment(C); { / }
      WriteComment(C2);{ * }
      Finish := False;
      while (not Finish) and (C2 <> #0) do begin
        C2 := GetChar;
        WriteComment(C2); { current comment char }
        if C2 = '*' then begin
          C2 := GetChar;
          if C2 = '/' then { end of comment } begin
            WriteComment(C2);
            Finish := True;
          end;
        end;
      end;
      C := GetChar; { for next pass }
      GetLexem := _COMMENT1;
    end
    else { OpSign } begin
      TokenValue := C;
      C := C2;
      GetLexem := _OPSIGN;
    end;
  end;

  procedure Literal;   { ' - literal }
  begin
    TokenValue := C;
    Finish := False;
    repeat
      C := GetChar;
      if C = #0 then begin
        GetLexem := _UNKNOWN;
        Exit;
      end;
      TokenValue := TokenValue + C;
      if C = '''' then begin
        C := GetChar;
        if C <> '''' then   { not double ' }
          Finish := True;
      end;
    until Finish;
    GetLexem := _LITERAL;
    C := GetChar;
  end;

  procedure Literal2;   { " - literal }
  begin
    TokenValue := C;
    Finish := False;
    repeat
      C := GetChar;
      if C = #0 then begin
        GetLexem := _UNKNOWN;
        Exit;
      end;
      if C = '"' then begin
        Finish := True;
      end;
      TokenValue := TokenValue + C;
    until Finish;
    if SQLDialect <> 3 then GetLexem := _LITERAL2
    else GetLexem := _IDENT;
    C := GetChar;
  end;

  procedure NELE;     { < - possible <> or <= }
  begin
    TokenValue := C;
    C := GetChar;
    if C = '>' then { <> } begin
      TokenValue := TokenValue + C;
      GetLexem := _NE;
      C := GetChar;
    end
    else
      if C = '=' then { <= } begin
        TokenValue := TokenValue + C;
        GetLexem := _LE;
        C := GetChar;
      end
      else             { < }
        GetLexem := _OPSIGN;
  end;

  procedure GE;       { > - possible >= }
  begin
    TokenValue := C;
    C := GetChar;
    if C = '=' then  { >= } begin
      TokenValue := TokenValue + C;
      GetLexem := _GE;
      C := GetChar;
    end
    else             { > }
      GetLexem := _OPSIGN;
  end;

  procedure Link;       { | - possible || }
  begin
    TokenValue := C;
    C := GetChar;
    if C = '|' then  { || } begin
      TokenValue := TokenValue + C;
      GetLexem := _11;
      C := GetChar;
    end
    else             { | }
      GetLexem := _UNKNOWN;
  end;

  procedure EOL;       { #13 - possible #13#10 }
  begin
    TokenValue := C;
    C := GetChar;
    if C = #10 then  { #13#10 } begin
      TokenValue := TokenValue + C;
      GetLexem := _EOL;
      C := GetChar;
    end
    else             { #13 }
      GetLexem := _UNKNOWN;
  end;

  procedure Number;   { digit - number }
  var
    HasDot: Boolean;
  begin
    TokenValue := C;
    HasDot := False;
    repeat
      C := GetChar;
      if (C = #0) or IsDelimiter(C) then begin
        GetLexem := _NUMBER;
        Exit;
      end;
      if IsDigit(C) then begin
        TokenValue := TokenValue + C;
      end
      else begin
        if (not HasDot) and (C = '.') then begin
          HasDot := True;
          TokenValue := TokenValue + C;
        end
        else begin
          TokenValue := TokenValue + C;
          C := GetChar;
          GetLexem := _UNKNOWN;
          Exit;
        end;
      end;
      until False;
  end;

  procedure Ident;    { alfa or _  - identifier }
  begin
    TokenValue := C;
    Finish := False;
    repeat
      C := GetChar;
      if C = #0 then
        Finish := True;
      if IsAlpha(C) or IsDigit(C) or (C = '_') or (C = '$') then begin
        TokenValue := TokenValue + C;
      end
      else
        Finish := True;
    until Finish;
    GetLexem := _IDENT;
    { keyword may be }
    {StrLoCase := LoCaseStr(TokenValue);
    if KeyWords.Find(StrLoCase, Index) then begin
      GetLexem := _KEYWORD;
      KeywordIndex := Index;
    end;}
  end;

  procedure OpSign;
  begin
    TokenValue := C;
    C := GetChar;
    GetLexem := _OPSIGN;
  end;

  procedure Delimiter;
  begin
    TokenValue := C;
    C := GetChar;
    GetLexem := _DELIMITER;
  end;

begin { GetLexem }

  KeywordIndex := -1;
  
  { skip spaces, TABs and CtrlZ ! }
  while (C = ' ') or (C = #9) or
        (C = #26) or (C = #13) or (C = #10) do begin
    C := GetChar;
  end;

  if C = #0 then { eof } begin
    GetLexem := 0;
    Exit;
  end;

  case C of
    '''': Literal;               { literal }
    '"':  Literal2;              { literal2 }
    '<':  NELE;                  { possible <> or <= }
    '>':  GE;                    { possible >= }
    '0'..'9': Number;            { number }
    'a'..'z',
    'A'..'Z',
    '_','^','!':  Ident;         { identifier or keyword }
    '|':  Link;                  { possible || }
    '/':  Comments1;             { Comment or divide sign }
    '+','-','*',
    '=':  OpSign;                { operator }
    '(',')','[',']','{','}',
    '.',',',';',':': Delimiter; { delimiter }
    #0: EndOfText;
    else
      ErrorProc;
  end;
end;

{ private }
function    TLex.IsDigit(Ch: Char): Boolean;
begin
  IsDigit := ('0' <= Ch) and (Ch <= '9');
end;

function    TLex.IsHexDigit(Ch: Char): Boolean;
var Tmp: char;
begin
  Tmp := UpCase(Ch);
  IsHexDigit := ('A' <= Tmp) and (Tmp <= 'F');
end;

function    TLex.IsAlpha(Ch: Char): Boolean;
begin
  IsAlpha := (('a' <= Ch) and (Ch <= 'z')) or
             (('A' <= Ch) and (Ch <= 'Z'));
end;

function    TLex.IsDelimiter(Ch: Char): Boolean;
begin
  Result :=
     (C = ' ') or (C = #9) or (C = ',') or (C = ')') or
     (C = '=') or (c = '+') or (C = '-') or (C = '*') or
     (C = '/') or (C = ';') or (C = '|') or
     (C = #26) or (C = #13) or (C = #10);
end;

procedure CreateKeywords;
begin
  KeyWords := TStringList.Create;
  KeyWords.Sorted := True;
  { Reserved Words }
  with KeyWords do begin
    {Add('active');
    Add('add');
    Add('after');
    Add('all');}
    Add('alter');
    {Add('and');
    Add('any');
    Add('as');
    Add('asc');
    Add('ascending');
    Add('at');
    Add('auto');
    Add('avg');
    Add('before');
    Add('begin');
    Add('between');
    Add('blob');
    Add('by');
    Add('cast');
    Add('char');
    Add('character');
    Add('check');
    Add('close');
    Add('collation');
    Add('collate');}
    Add('column');
    Add('commit');
    {Add('computed');}
    Add('constraint');
    {Add('containing');
    Add('continue');
    Add('count');}
    Add('create');
    {Add('cstring');
    Add('current');}
    Add('database');
    {Add('date');
    Add('decimal');}
    Add('declare');
    {Add('default');
    Add('delete');
    Add('descending');
    Add('distinct');
    Add('do');}
    Add('documentation');
    Add('domain');
    {Add('double');}
    Add('drop');
    {Add('else');
    Add('end');
    Add('entry_point');
    Add('event');}
    Add('exception');
    {Add('execute');
    Add('exists');}
    Add('exit');
    Add('external');
    {Add('file');
    Add('float');
    Add('for');
    Add('foreign');
    Add('from');
    Add('full');
    Add('function');}
    Add('generator');
    {Add('goto');
    Add('grant');
    Add('group');
    Add('having');
    Add('if');
    Add('immediate');
    Add('in');
    Add('inactive');
    Add('index');
    Add('init');
    Add('inner');
    Add('insert');
    Add('integer');
    Add('into');
    Add('is');
    Add('join');
    Add('key');
    Add('left');
    Add('like');
    Add('max');
    Add('min');
    Add('module_name');
    Add('not');
    Add('null');
    Add('numeric');}
    Add('off');
    Add('on');
    {Add('or');
    Add('order');
    Add('outer');
    Add('plan');
    Add('post_event');
    Add('precision');}
    Add('procedure');
    {Add('primary');
    Add('public');
    Add('references');
    Add('returns');
    Add('returning_values');
    Add('revoke');
    Add('right');}
    Add('rollback');
    {Add('select');}
    Add('set');
    {Add('smallint');
    Add('sort');
    Add('starting');
    Add('sum');
    Add('suspend');}
    Add('table');
    Add('term');
    {Add('then');
    Add('to');}
    Add('trigger');
    {Add('union');
    Add('unique');
    Add('update');
    Add('user');
    Add('values');
    Add('varchar');
    Add('variable');
    Add('varying');}
    Add('view');
    {Add('wait');
    Add('when');
    Add('where');
    Add('while');
    Add('with');}
    Add('work');

    //      
    IND_SET := IndexOf('set');
    IND_COMMIT := IndexOf('commit');
    IND_ROLLBACK := IndexOf('rollback');
    IND_CREATE := IndexOf('create');
    IND_DROP := IndexOf('drop');
    IND_DISCONNECT := IndexOf('disconnect');
    IND_CONNECT := IndexOf('connect');
    IND_EXIT := IndexOf('exit');
    IND_DOMAIN := IndexOf('domain');
    IND_TABLE := IndexOf('table');
    IND_VIEW := IndexOf('view');
    IND_PROCEDURE := IndexOf('procedure');
    IND_TRIGGER := IndexOf('trigger');
    IND_GENERATOR := IndexOf('generator');
    IND_EXCEPTION := IndexOf('exception');
    IND_EXTERNAL := IndexOf('external');
    IND_ALTER := IndexOf('alter');
    IND_DECLARE := IndexOf('declare');
    IND_ADD := IndexOf('add');
    IND_CONSTRAINT := IndexOf('constraint');
    IND_AUTODDL := IndexOf('autoddl');
    IND_OFF := IndexOf('off');
    IND_ON := IndexOf('on');
    IND_WORK := IndexOf('work');
    IND_DOCUMENTATION := IndexOf('documentation');
    IND_COLUMN := IndexOf('column');
    IND_NAMES := IndexOf('names');
    IND_TERM := IndexOf('term');
    IND_DATABASE := IndexOf('database');
  end;

end;

procedure DestroyKeywords;
begin
  Keywords.Clear;
  Keywords.Free;
  Keywords := nil;
end;

initialization
  Keywords := nil;
finalization
  if Keywords <> nil then DestroyKeywords;
end.
