{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
unit SoIBManager;

{$I SOHOIBLIB.INC}

interface

uses Windows, Messages, SysUtils, Classes, SoIBClasses, FIB;

type

  EIBError = EFIBError;

  {      }
  TsohoIbConnectType = (ctLocal, ctTCPIP, ctNetBeui, ctIPX);

  TstrIBSystem = string;
  TstrResult = string;

  PibMetaRecord = ^TibMetaRecord;
  {        
         
    TsohoIBSQL,   ,   
     }
  TibMetaRecord = record
    Database: TsohoIBDatabase;
    Transaction: TsohoIBTransaction;
    Query: TsohoIBSQL;
    UserData: pointer;
  end;

  {   ,  "  "
        ,    
       -  
  }
  TsohoIBManager = class(TComponent)
  private
    { Private declarations }
    FDatabases: TList;
    FLastSQLErrorCode : LongInt;
    FLastSQlErrorText : TstrResult;
    FLastSQLErrorPos  : TPoint;
  protected
    { Protected declarations }
    procedure SetUserData(Index: Integer; const Value: pointer);
    function GetUserData(Index: LongInt): pointer;
    function GetDatabase(Index: LongInt): TsohoIBDatabase;
    function GetQuery(Index: LongInt): TsohoIBSQL;
    function GetTransaction(Index: LongInt): TsohoIBTransaction;
    procedure SetLastError(E: EIBError);
    function GetObjectsList(aDatabase: TsohoIBDatabase; List, Desc : TStrings) : boolean;
    function ExecMetaQuery(aDatabase: TsohoIBDatabase) : boolean;
  public
    {      ,  aDatabase  
       .   aDatabase   ,    
      aDatabase }
    function GetSelectedDatabase(aDatabase: TsohoIBDatabase): TsohoIBDatabase;
    {      }
    function IndexOfDatabase(aDatabase: TsohoIBDatabase): LongInt;
    {     }
    function GetDomains(aDatabase: TsohoIBDatabase; List, Desc : TStrings; ShowSystem : boolean) : boolean;
    {     }
    function GetTables(aDatabase: TsohoIBDatabase; List, Desc : TStrings; ShowSystem : boolean) : boolean;
    {    ,  RDB$RELATION_ID  List.Items[].Data }
    function GetTablesWithId(aDatabase: TsohoIBDatabase; List : TStrings; ShowSystem : boolean) : boolean;
    {     }
    function GetViews(aDatabase: TsohoIBDatabase; List, Desc : TStrings) : boolean;
    {      }
    function GetProcedures(aDatabase: TsohoIBDatabase; List, Desc : TStrings) : boolean;
    {     }
    function GetTriggers(aDatabase: TsohoIBDatabase; List, Desc : TStrings; ShowSystem : boolean) : boolean;
    {     }
    function GetTriggersForTable(aDatabase: TsohoIBDatabase; const TableName: TstrIBSystem; List, Desc : TStrings; ShowSystem : boolean) : boolean;
    {     }
    function GetIndexesForTable(aDatabase: TsohoIBDatabase; const TableName: TstrIBSystem; List, Desc : TStrings) : boolean;
    {     }
    function GetGenerators(aDatabase: TsohoIBDatabase; List : TStrings) : boolean;
    {     }
    function GetExceptions(aDatabase: TsohoIBDatabase; List, Desc : TStrings) : boolean;
    {      }
    function GetUDFs(aDatabase: TsohoIBDatabase; List, Desc : TStrings) : boolean;
    {     }
    function  GetFields(aDatabase: TsohoIBDatabase; const TableName : TstrIBSystem; FieldsList : TStrings) : boolean;
    {      NewValue }
    function SetGeneratorValue (aDatabase: TsohoIBDatabase; const GeneratorName : TstrIBSystem; NewValue : LongInt) : boolean;
    {     }
    function GetGeneratorValue (aDatabase: TsohoIBDatabase; const GeneratorName : TstrIBSystem) : LongInt;
    {          }
    function GetDataBaseNameInfo (aDatabase: TsohoIBDatabase; var ServerName, FileName : string; var Protocol : TsohoIbConnectType) : boolean;
    {         , 
           }
    function GetDataBaseNameInfoByName (const aDatabaseName: string; var ServerName, FileName : string; var Protocol : TsohoIbConnectType) : boolean;
    {    }
    function ProtocolToStr (Protocol : TsohoIbConnectType) : TstrResult;

    {    }
    procedure RegisterDatabase(aDatabase: TsohoIBDatabase); virtual;
    {    .   
       .  aDatabase <> nil,    
      ,  Index <> -1,     
         }
    procedure UnregisterDatabase(aDatabase: TsohoIBDatabase; Index: LongInt); virtual;
    {      .  AddNumbers ,
                  }
    procedure GetDatabaseNames(Strings: TStrings; AddNumbers: boolean {$IFDEF Delphi4_or_5}= true{$ENDIF});
    {    , ,    
              }
    procedure SetConnectParams(const aUserName, aPassword, aRole, aConnectCharSet: string);

    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;

    {     }
    procedure Clear;
    { -    }
    function DatabasesCount: LongInt;
    {     }
    property Databases[Index: LongInt]: TsohoIBDatabase read GetDatabase;
    {   TsohoIBSQL     }
    property Queries[Index: LongInt]: TsohoIBSQL read GetQuery;
    {   TsohoIBTransaction     }
    property Transactions[Index: LongInt]: TsohoIBTransaction read GetTransaction;
    {   ,     }
    property UserData[Index: LongInt]: pointer read GetUserData write SetUserData;
    {     SQL }
    property LastSQLErrorCode: LongInt read FLastSQLErrorCode;
    {     SQL }
    property LastSQLErrorText: TstrResult read FLastSQLErrorText;
    {       SQL }
    property LastSQLErrorPos: TPoint read FLastSQLErrorPos;
  published
  end;

  TsohoDBList = class(TList)
  private
    function GetDatabase(Index: Integer): TsohoIBDataBase;
    procedure SetDatabase(Index: Integer; const Value: TsohoIBDataBase);
  public
    procedure AddDatabase(aOwner: TComponent; const aDatabaseName, aUserName, aPassword,
      aRole, aConnectCharSet: string);
    procedure CreateDatabaseList(aOwner: TComponent; const aUserName, aPassword,
      aRole, aConnectCharSet: string; Items: TStrings);
    procedure Close; virtual;  
{$IFDEF Delphi3}	
    procedure ClearItems;
{$ELSE}
    procedure Clear; override;
{$ENDIF}
    destructor Destroy; override;
    property Database[Index: LongInt]: TsohoIBDataBase read GetDatabase write SetDatabase;
  end;

var
  {     TsohoIBManager,      }
  ActiveManager: TsohoIBManager;

implementation
uses SoVCLUtils, SoAppUtils, SoIBUtil, SoStrUtils, SoIBCns;

{ TsohoIBManager }
constructor TsohoIBManager.Create(aOwner: TComponent);
begin
  inherited;
  FDatabases := TList.Create;
  if ActiveManager = nil then
    ActiveManager := Self;
end;

destructor TsohoIBManager.Destroy;
begin
  Clear;
  FDatabases.Free;
  if ActiveManager = Self then
    ActiveManager := nil;
  inherited;
end;

function TsohoIBManager.GetDatabase(Index: Integer): TsohoIBDatabase;
begin
  Result := PibMetaRecord(FDatabases[Index])^.Database;
end;

function TsohoIBManager.GetDataBaseNameInfoByName (const aDatabaseName: string; var ServerName,
   FileName : string; var Protocol : TsohoIbConnectType) : boolean;
var Tmp, Temp : string;
begin
  Tmp := aDatabaseName;
  ServerName := 'LIB';
  FileName   := aDatabaseName;
  if Tmp[2] = ':' then begin Protocol := ctLocal; end
  else begin
    // netBEUI Connection...
    if Pos('\\', Tmp) <> 0 then begin
      Temp     := Copy(Tmp, 3, Length(Tmp) - 2);
      ServerName   := Copy(Temp, 1, Pos('\', Temp) - 1);
      FileName := Copy(Temp, Pos('\', Temp) + 1, Length(Temp) - Pos('\', Temp));
      Protocol   := ctNetBeui;
    end;
    // TCP/IP Connection...
    if (Pos('@', Tmp) = 0) and (Pos('\\', Tmp) = 0) and
       (Pos(':', Tmp) <> 0) then begin
      ServerName   := Copy(Tmp, 1, Pos(':', Tmp) - 1);
      FileName := Copy(Tmp, Pos(':', Tmp) + 1, Length(Tmp) - Pos(':', Tmp));
      Protocol   := ctTCPIP;
    end;
    // IPX Connection...
    if Pos('@', Tmp) <> 0  then begin
      ServerName   := Copy(Tmp, 1, Pos('@', Tmp) - 1);
      FileName := Copy(Tmp, Pos('@', Tmp) + 1, Length(Tmp) - Pos('@', Tmp));
      Protocol := ctIPX;
    end;
  end;
  Result := true;
end;

function TsohoIBManager.GetDataBaseNameInfo(aDatabase: TsohoIBDatabase;
  var ServerName, FileName: string; var Protocol: TsohoIbConnectType): boolean;
var FDB: TsohoIBDatabase;
begin
  FDB := GetSelectedDatabase(aDatabase);
  if (FDB = nil) or (not FDB.Connected) then begin
    Result := true;
    ErrorMsg(sohoIBNoConnection);
    exit;
  end;
  Result := GetDataBaseNameInfoByName(trim(FDB.DatabaseName), ServerName,
    FileName, Protocol);
end;

function TsohoIBManager.GetObjectsList (aDatabase: TsohoIBDatabase;
  List, Desc : TStrings) : boolean;
var Index: LongInt;
begin
  Index := IndexOfDatabase(aDatabase);
  Result := (aDatabase <> nil) and (Index <> -1);
  if not Result then exit;
  PushCursor(crHourGlass);

  with Queries[Index] , SQL do begin
    if not NewTransactionStart(Transactions[Index]) then exit;
    try
      List.Clear;
      if Desc <> nil then Desc.Clear;
      ExecSQL;
      while not Eof do begin
        List.Add(trim(Fields[0].AsString));
        if (Desc <> nil) and (Current.Count > 1) then
          Desc.Add(trim(Fields[1].AsString));
        Next;
      end;
      Transactions[Index].Commit;
    except
      on E: EIBError do begin
        Transactions[Index].Rollback;
        SetLastError (E);
        Result := false;
        ErrorMsg(E.Message);
        if E.SQLCode = - 901 then exit;
      end;
    end;
    Close;
  end;
  RestoreCursor;
end;

function TsohoIBManager.GetDomains(aDatabase: TsohoIBDatabase; List,
  Desc: TStrings; ShowSystem: boolean): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;
  if not ShowSystem then begin              
    with Query.SQL do begin
      Add('select RDB$FIELD_NAME, RDB$DESCRIPTION from RDB$FIELDS');
      Add('where (RDB$SYSTEM_FLAG <> 1) and not (RDB$FIELD_NAME starting with '+
        SingleQuotes('RDB$') + ')');
      Add('order by RDB$FIELD_NAME');
    end;    // with
  end
  else begin
    with Query.SQL do begin
      Add('select RDB$FIELD_NAME, RDB$DESCRIPTION from RDB$FIELDS');
      Add('order by RDB$FIELD_NAME');
    end;    // with
  end;
  Result := GetObjectsList(FDB, List, Desc);
end;

function TsohoIBManager.GetExceptions(aDatabase: TsohoIBDatabase; List,
  Desc: TStrings): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;
  with Query.SQL do begin
    Add('select RDB$EXCEPTION_NAME,RDB$DESCRIPTION from RDB$EXCEPTIONS where (RDB$SYSTEM_FLAG = 0) or ');
    Add('(RDB$SYSTEM_FLAG is null) order by RDB$EXCEPTION_NAME');
  end;    // with
  Result := GetObjectsList(FDB, List, Desc);
end;

function TsohoIBManager.GetFields(aDatabase: TsohoIBDatabase;
  const TableName: TstrIBSystem; FieldsList: TStrings): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;
  with Query.SQL do begin
    Add('select RDB$FIELD_NAME from RDB$RELATION_FIELDS');
    Add('where RDB$RELATION_NAME = ' + SingleQuotes(TableName));
    Add('order by RDB$FIELD_POSITION');
  end;    // with
  Result := GetObjectsList(FDB, FieldsList, nil);
end;

function TsohoIBManager.GetGenerators(aDatabase: TsohoIBDatabase;
  List: TStrings): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;
  with Query.SQL do begin
    Add('select RDB$GENERATOR_NAME from RDB$GENERATORS where (RDB$SYSTEM_FLAG = 0) or ');
    Add('(RDB$SYSTEM_FLAG is null) order by RDB$GENERATOR_NAME');
  end;    // with
  Result := GetObjectsList(FDB, List, nil);
end;

function TsohoIBManager.GetGeneratorValue(aDatabase: TsohoIBDatabase;
  const GeneratorName: TstrIBSystem): LongInt;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  Result := 0;
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  if (FDB <> nil) and (Index <> -1) then exit;
  Query := Queries[Index];
  PushCursor(crHourGlass);

  Query.SQL.Clear;

  if not NewTransactionStart(Transactions[Index]) then exit;
  with Query, SQL do begin
    try
      Add('select Gen_Id(' +
        FormatIdentifier(FDB.SQLDialect, GeneratorName) + ', 0) from RDB$DATABASE');
      ExecSQL;
      Result := Fields[0].AsInteger;
      Transactions[Index].Commit;
    except
      on E: EIBError do begin
        Transactions[Index].Rollback;
        SetLastError(E);
      end;
    end;
    Close;
  end;
  RestoreCursor;
end;

function TsohoIBManager.GetIndexesForTable(aDatabase: TsohoIBDatabase;
  const TableName: TstrIBSystem; List, Desc: TStrings): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;
  with Query.SQL do begin
    Add('select RDB$INDEX_NAME, RDB$DESCRIPTION from RDB$INDICES');
    Add('where (RDB$RELATION_NAME = ' +
      SingleQuotes(TableName) + ')');
  end;    // with
  Result := GetObjectsList(FDB, List, Desc);
end;

function TsohoIBManager.GetProcedures(aDatabase: TsohoIBDatabase; List,
  Desc: TStrings): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;
  Query.SQL.Add('select RDB$PROCEDURE_NAME, RDB$DESCRIPTION from RDB$PROCEDURES order by RDB$PROCEDURE_NAME');
  Result := GetObjectsList(FDB, List, Desc);
end;

function TsohoIBManager.GetQuery(Index: Integer): TsohoIBSQL;
begin
  Result := PibMetaRecord(FDatabases[Index])^.Query;
end;

function TsohoIBManager.GetSelectedDatabase(
  aDatabase: TsohoIBDatabase): TsohoIBDatabase;
begin
  Result := aDatabase;
  if (Result = nil) and (FDatabases.Count > 0) then
    Result := Databases[0];
end;

function TsohoIBManager.GetTables(aDatabase: TsohoIBDatabase; List,
  Desc: TStrings; ShowSystem: boolean): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;
  if not ShowSystem then begin
    with Query.SQL do begin
      Add('select RDB$RELATION_NAME, RDB$DESCRIPTION from RDB$RELATIONS where (RDB$SYSTEM_FLAG = 0)');
      Add('and (RDB$VIEW_SOURCE is Null) and not (RDB$RELATION_NAME STARTING WITH ' +
       SingleQuotes('QDGRANT$') + ')');
      Add('order by RDB$RELATION_NAME');
    end;    // with
  end
  else begin
    with Query.SQL do begin
      Add('select RDB$RELATION_NAME, RDB$DESCRIPTION from RDB$RELATIONS where ');
      Add('(RDB$VIEW_SOURCE is Null) and not (RDB$RELATION_NAME STARTING WITH ' +
        SingleQuotes('QDGRANT$') + ')');
      Add('order by RDB$RELATION_NAME');
    end;    // with
  end;
  Result := GetObjectsList(FDB, List, Desc);
end;

function TsohoIBManager.GetTablesWithId(aDatabase: TsohoIBDatabase;
  List: TStrings; ShowSystem: boolean): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;

  with Query, SQL do begin
    if not NewTransactionStart(Transactions[Index]) then exit;
    try
      if not ShowSystem then begin
        Add('select RDB$RELATION_NAME, RDB$RELATION_ID from RDB$RELATIONS where (RDB$SYSTEM_FLAG = 0)');
        Add('and (RDB$VIEW_SOURCE is Null) and not (RDB$RELATION_NAME STARTING WITH '+
        SingleQuotes('QDGRANT$')+ ') ');
        Add('order by RDB$RELATION_ID');
      end
      else begin
        Add('select RDB$RELATION_NAME, RDB$RELATION_ID from RDB$RELATIONS where ');
        Add('(RDB$VIEW_SOURCE is Null) and not (RDB$RELATION_NAME STARTING WITH '+
          SingleQuotes('QDGRANT$') + ')');
        Add('order by RDB$RELATION_ID');
      end;
      List.Clear;
      ExecSQL;
      while not Eof do begin
        List.AddObject(trim(Fields[0].AsString), pointer(Fields[1].AsInteger));
        Next;
      end;
      Transactions[Index].Commit;
      Result := true;
    except
      on E: EIBError do begin
        Transactions[Index].Rollback;
        SetLastError (E);
        Result := false;
        ErrorMsg(E.Message);
        if E.SQLCode = - 901 then exit;
      end;
    end;
    Close;
  end;
  RestoreCursor;
end;

function TsohoIBManager.GetTransaction(Index: Integer): TsohoIBTransaction;
begin
  Result := PibMetaRecord(FDatabases[Index])^.Transaction;
end;

function TsohoIBManager.GetTriggers(aDatabase: TsohoIBDatabase; List,
  Desc: TStrings; ShowSystem: boolean): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;
  if ShowSystem then begin
    with Query.SQL do begin
      Add('select RDB$TRIGGER_NAME, RDB$DESCRIPTION from RDB$TRIGGERS where (RDB$SYSTEM_FLAG = 0) or');
      Add('(RDB$SYSTEM_FLAG is null) order by RDB$TRIGGER_NAME');
    end;    // with
  end
  else begin
    with Query.SQL do begin
      Add('select RDB$TRIGGERS.RDB$TRIGGER_NAME, RDB$TRIGGERS.RDB$DESCRIPTION from RDB$TRIGGERS');
      Add('where ((RDB$TRIGGERS.RDB$SYSTEM_FLAG = 0) or (RDB$TRIGGERS.RDB$SYSTEM_FLAG is null)) and');
      Add('(select count(*) from RDB$CHECK_CONSTRAINTS where RDB$TRIGGER_NAME = RDB$TRIGGERS.RDB$TRIGGER_NAME) = 0');
      Add('order by RDB$TRIGGER_NAME');
    end;    // with
  end;
  Result := GetObjectsList(FDB, List, Desc);
end;

function TsohoIBManager.GetTriggersForTable(aDatabase: TsohoIBDatabase;
  const TableName: TstrIBSystem; List, Desc: TStrings;
  ShowSystem: boolean): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
    Tmp: string;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;

  Tmp := TableName;
  if ShowSystem then begin
    with Query.SQL do begin
      Add('select RDB$TRIGGER_NAME, RDB$DESCRIPTION from RDB$TRIGGERS where (RDB$RELATION_NAME ='+
        SingleQuotes(Tmp) + ')');
      Add('and ((RDB$SYSTEM_FLAG = 0) or (RDB$SYSTEM_FLAG is null))');
      Add('order by RDB$TRIGGER_NAME');
    end;    // with
  end
  else begin
    with Query.SQL do begin
      Add('select RDB$TRIGGERS.RDB$TRIGGER_NAME, RDB$TRIGGERS.RDB$DESCRIPTION from RDB$TRIGGERS');
      Add('where (RDB$RELATION_NAME = ' + SingleQuotes(Tmp) + ') and ');
      Add('((RDB$TRIGGERS.RDB$SYSTEM_FLAG = 0) or (RDB$TRIGGERS.RDB$SYSTEM_FLAG is null)) and');
      Add('(select count(*) from RDB$CHECK_CONSTRAINTS where RDB$TRIGGER_NAME = RDB$TRIGGERS.RDB$TRIGGER_NAME) = 0');
      Add('order by RDB$TRIGGER_NAME');
    end;    // with
  end;
  Result := GetObjectsList(FDB, List, Desc);
end;

function TsohoIBManager.GetUDFs(aDatabase: TsohoIBDatabase; List,
  Desc: TStrings): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;
  with Query.SQL do begin
    Clear;
    Add('select RDB$FUNCTION_NAME, RDB$DESCRIPTION from RDB$FUNCTIONS order by RDB$FUNCTION_NAME');
  end;    // with
  Result := GetObjectsList(FDB, List, Desc);
end;

function TsohoIBManager.GetViews(aDatabase: TsohoIBDatabase; List,
  Desc: TStrings): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;
  with Query.SQL do begin
    Clear;
    Add('select RDB$RELATION_NAME, RDB$DESCRIPTION from RDB$RELATIONS where not(RDB$VIEW_SOURCE is Null)');
    Add('order by RDB$RELATION_NAME');
  end;    // with
  Result := GetObjectsList(FDB, List, Desc);
end;

function TsohoIBManager.IndexOfDatabase(
  aDatabase: TsohoIBDatabase): LongInt;
var Index: LongInt;
begin
  Result := -1;
  for Index := 0 to pred(FDatabases.Count) do
    if PibMetaRecord(FDatabases[Index])^.Database = aDatabase then begin
      Result := Index;
      exit;
    end;
end;

function TsohoIBManager.ProtocolToStr(
  Protocol: TsohoIbConnectType): TstrResult;
begin
  case Protocol of    //
    ctLocal    : Result := 'Local';
    ctTCPIP    : Result := 'TCP/IP';
    ctNetBeui  : Result := 'NETBEUI';
    ctIPX      : Result := 'IPX';
  end;    // case
end;

procedure TsohoIBManager.RegisterDatabase(aDatabase: TsohoIBDatabase);
var Tmp: PibMetaRecord;
begin
  New(Tmp);
  Tmp.Database := aDatabase;

  Tmp.Transaction := TsohoIBTransaction.Create(nil);
  Tmp.Query := TsohoIBSQL.Create(nil);

  with Tmp^ do begin
    Transaction.DefaultDatabase := aDatabase;
    Query.Transaction := Transaction;
    Query.Database := aDatabase;
    UserData := nil;
  end;

  FDatabases.Add(Tmp);
end;

function TsohoIBManager.ExecMetaQuery(aDatabase: TsohoIBDatabase) : boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;

  PushCursor(crHourGlass);
  if not NewTransactionStart(Transactions[Index]) then exit;
  try
    Queries[Index].ExecQuery;
    Result := true;
    Transactions[Index].Commit;
  except
    on E: EIBError do begin
      SetLastError (E);
      Transactions[Index].Rollback;
      Result := false;
    end;
  end;
  RestoreCursor;
end;

function TsohoIBManager.SetGeneratorValue(aDatabase: TsohoIBDatabase;
  const GeneratorName: TstrIBSystem; NewValue: Integer): boolean;
var FDB: TsohoIBDatabase;
    Index: LongInt;
    Query: TsohoIBSQL;
begin
  FDB := GetSelectedDatabase(aDatabase);
  Index := IndexOfDatabase(FDB);
  Result := (FDB <> nil) and (Index <> -1);
  if not Result then exit;
  Query := Queries[Index];

  Query.SQL.Clear;
  Query.SQL.Add('set generator ' +
    FormatIdentifier(FDB.SQLDialect, GeneratorName) + ' to ' + IntToStr(NewValue));
  Result := ExecMetaQuery(FDB);
end;

procedure TsohoIBManager.SetLastError(E: EIBError);
var Trans: TsohoIBErrorPos;
begin
  FLastSQLErrorCode := E.SQLCode;
  FLastSQLErrorText := E.Message;
  Trans := TranslateIBError(E.Message);
  FLastSQLErrorPos.X := Trans.Char;
  FLastSQLErrorPos.Y := Trans.Line;
end;

procedure TsohoIBManager.UnregisterDatabase(aDatabase: TsohoIBDatabase;
  Index: Integer);

  procedure DeleteByIndex(aIndex: LongInt);
  begin
    if aIndex = -1 then exit;
    PibMetaRecord(FDatabases[aIndex])^.Query.Free;
    PibMetaRecord(FDatabases[aIndex])^.Transaction.Free;
    FreeMem(FDatabases[aIndex]);
    FDatabases.Delete(aIndex);
  end;

begin
  if (aDatabase = nil) and (Index = -1) then exit;
  if Index <> -1 then DeleteByIndex(Index)
  else DeleteByIndex(IndexOfDatabase(aDatabase));
end;

function TsohoIBManager.DatabasesCount: LongInt;
begin
  Result := FDatabases.Count;
end;

procedure TsohoIBManager.GetDatabaseNames(Strings: TStrings;
  AddNumbers: boolean {$IFDEF Delphi4_or_4}= true{$ENDIF});
var Index: LongInt;
    ServerName, DBName, Item: string;
    Protocol: TsohoIbConnectType;
begin
  Strings.Clear;
  for Index := 0 to pred(DatabasesCount) do begin
    GetDataBaseNameInfo(Databases[Index], ServerName, DBName,
      Protocol);
    Item := EmptyStr;
    if Protocol <> ctLocal then
      Item := '(' + ActiveManager.ProtocolToStr(Protocol) + ')';
    Item := ExtractFileName(DBName) + ' on ' + ServerName + Space
     + Item;
    if AddNumbers then Item := IntToStr(Index + 1) + ' - '
      + Item;
    Strings.Add(Item);
  end;
end;

procedure TsohoIBManager.SetConnectParams(const aUserName, aPassword,
  aRole, aConnectCharSet: string);
var Index: LongInt;
begin
  for Index := 0 to pred(DatabasesCount) do
    with Databases[Index] do begin
      if Connected then Continue;
      UserName := aUserName;
      Password := aPassword;
      Role := aRole;
      ConnectCharSet := aConnectCharSet;
    end;
end;

procedure TsohoIBManager.SetUserData(Index: Integer; const Value: pointer);
begin
  PibMetaRecord(FDatabases[Index])^.UserData := Value;
end;

function TsohoIBManager.GetUserData(Index: Integer): pointer;
begin
  Result := PibMetaRecord(FDatabases[Index])^.UserData;
end;

procedure TsohoIBManager.Clear;
var Index: LongInt;
begin
  for Index := pred(FDatabases.Count) downto 0 do
    UnregisterDatabase(nil, Index);
end;

{ TsohoDBList }
procedure TsohoDBList.AddDatabase(aOwner: TComponent;
  const aDatabaseName, aUserName, aPassword, aRole,
  aConnectCharSet: string);
var Tmp: TsohoIBDatabase;
begin
  Tmp := TsohoIBDatabase.Create(aOwner);
  with Tmp do begin
    DatabaseName := aDatabaseName;
    UserName := aUserName;
    Password := aPassword;
    Role := aRole;
    ConnectCharSet := aConnectCharSet;
    Add(Tmp);
  end;
end;

{$IFDEF Delphi3}
procedure TsohoDBList.ClearItems;
{$ELSE}
procedure TsohoDBList.Clear;
{$ENDIF}
var Index: LongInt;
begin
  for Index := pred(Count) downto 0 do Database[Index].Free;
{$IFDEF Delphi3}
  Clear;
{$ELSE}
  inherited;
{$ENDIF}
end;

procedure TsohoDBList.Close;
var Index: LongInt;
begin
  for Index := pred(Count) downto 0 do
    Database[Index].Close;
end;

procedure TsohoDBList.CreateDatabaseList(aOwner: TComponent;
  const aUserName, aPassword, aRole,
  aConnectCharSet: string; Items: TStrings);
var Index: LongInt;
begin
  for Index := 0 to pred(Items.Count) do
    AddDatabase(aOwner, Items[Index], aUserName, aPassword, aRole, aConnectCharSet);
end;

destructor TsohoDBList.Destroy;
begin
  Clear;
  inherited;
end;

function TsohoDBList.GetDatabase(Index: Integer): TsohoIBDataBase;
begin
  Result := TsohoIBDataBase(Items[Index]);
end;

procedure TsohoDBList.SetDatabase(Index: Integer;
  const Value: TsohoIBDataBase);
begin
  Items[Index] := Value;
end;

initialization
  ActiveManager := nil;
end.
