{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{              Written by Igor Chernyshov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
{
  06.11.2000 -   TsohoIBGeneratorNameProperty -   
}
unit SoIBPrp;

{$I SOHOIBLIB.INC}
{$I FIBPLUS.INC}

interface
uses  Classes, {$IFDEF Delphi6}DesignEditors, DesignIntf{$ELSE}DsgnIntf{$ENDIF};

type
//------------------------ Garik
  TsohoIBTableNameProperty = class(TStringProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TsohoIBGeneratorNameProperty = class(TStringProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TsohoFormIBFieldsProperty = class(TStringProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TsohoFormIBGroupFieldsProperty = class(TsohoFormIBFieldsProperty)
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;

implementation
uses SysUtils, SoAppUtils, SoVCLUtils, SoSTRUtils, SoIBClasses, SohoIBAssist, SoIBFolder, SoIBTreeView,
     SoIBTVCont, SoIBGuide;

Var
  DBase     : TsohoIBDatabase;
  TName     : string;
  GName     : string;

//-----------------------------------------------------------------------------
// Common procedures
//-----------------------------------------------------------------------------
// Open database. Get Database property from Component
function Open_Database(Component : TPersistent) : boolean;
begin
  Result := False;

  // Component is TsohoIBAssist
  if (Component is TsohoIBAssist) then begin
    if (Component as TsohoIBAssist).DataSet = nil then Exit;
    TName := ((Component as TsohoIBAssist).TableName);
    DBase := TsohoIBDatabase(((Component as TsohoIBAssist).DataSet as TsohoIBDataSet).Database);
  end; {--- if ---}

  // Component is TsohoIBFolder
  if (Component is TsohoIBFolder) then begin
    DBase := TsohoIBDatabase((Component as TsohoIBFolder).Database);
  end; {--- if ---}

  // Component is TsohoIBTreeView
  if (Component is TsohoIBTreeView) then begin
    DBase     := TsohoIBDatabase((Component as TsohoIBTreeView).Database);
    TName     := ((Component as TsohoIBTreeView).ItemsTable);
    GName     := ((Component as TsohoIBTreeView).GroupsTable);
  end; {--- if ---}

  // Component is TsohoIBTVContainer
  if (Component is TsohoIBTVContainer) then begin
    DBase     := TsohoIBDatabase((Component as TsohoIBTVContainer).Database);
    TName     := ((Component as TsohoIBTVContainer).ItemsTable);
    GName     := ((Component as TsohoIBTVContainer).GroupsTable);
  end; {--- if ---}

  // Component is TsohoIBGuide
  if (Component is TsohoIBGuide) then begin
    DBase     := TsohoIBDatabase((Component as TsohoIBGuide).Database);
    TName     := ((Component as TsohoIBGuide).FolderTable);
  end; {--- if ---}

  if DBase = nil then Exit;
  if DBase.DefaultTransaction = nil then Exit;
  if not DBase.Connected then
    try
      DBase.Open;
    except
      raise; 
      exit;
    end;    { try/except }
  if DBase.DefaultTransaction = nil then begin
    ErrorMsg('No default Transaction component at Database component');
    exit; 
  end; {--- if ---}
  if not DBase.DefaultTransaction.Active then
    try
      { statements to try }
      DBase.DefaultTransaction.StartTransaction;
    except
      raise; 
      exit;
    end;    { try/except }
  Result := True;
end;

function CreateFieldsList (const TableName : string) : TStringList;
var
  Q : TsohoIBSQL;
begin
  q := TsohoIBSQL.Create(nil);
  PushCursor(crHourGlass);
  try
    q.Database := DBase;
    q.Transaction := DBase.DefaultTransaction;
    q.SQL.Add('select RDB$RELATION_FIELDS.RDB$FIELD_NAME as FIELDNAME');
    q.SQL.Add('from RDB$RELATION_FIELDS');
    q.SQL.Add('where RDB$RELATION_FIELDS.RDB$RELATION_NAME = '+SingleQuotes(TableName));
    q.SQL.Add('order by RDB$RELATION_FIELDS.RDB$FIELD_POSITION');
    q.ExecSQL;
    Result := TStringList.Create;
      while not q.eof do begin
        {$IFNDEF FIELD_BY_NAME_AS_FUNC}
        Result.Add(TrimRight(q.FieldByName['FIELDNAME'].AsString));
        {$ELSE}
        Result.Add(TrimRight(q.FieldByName('FIELDNAME').AsString));
        {$ENDIF}
        q.Next;
      end;
  finally
    q.Close;
    RestoreCursor;
  end; {--- try/finally ---}
end;

//-----------------------------------------------------------------------------
// TsohoIBTableNameProperty
//-----------------------------------------------------------------------------
function TsohoIBTableNameProperty.GetAttributes : TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TsohoIBTableNameProperty.GetValues(Proc : TGetStrProc);
Var
  Q : TsohoIBSQL;
Begin
  if not Open_Database(GetComponent(0)) then Exit;
  q := TsohoIBSQL.Create(nil);
  try
    q.Database := DBase;
    q.Transaction := DBase.DefaultTransaction;
    q.SQL.Add('select RDB$RELATION_NAME as TableName, RDB$DESCRIPTION from RDB$RELATIONS where (RDB$SYSTEM_FLAG = 0)');
    q.SQL.Add('and (RDB$VIEW_SOURCE is Null)');
    q.SQL.Add('order by RDB$RELATION_NAME');
    q.ExecSQL;
    while not (Q.EOF) do begin
      Proc(Trim(q.Fields[0].AsString));
      q.Next;
    end; {--- while ---}
  finally
    q.Transaction.Commit;
    q.Close;
    q.Free;
  end; {--- try/finally ---}
end;

//-----------------------------------------------------------------------------
// TsohoIBGeneratorNameProperty
//-----------------------------------------------------------------------------
function TsohoIBGeneratorNameProperty.GetAttributes : TPropertyAttributes;
begin
  Result := [paValueList];
end;

procedure TsohoIBGeneratorNameProperty.GetValues(Proc : TGetStrProc);
Var
  Q      : TsohoIBSQL;
Begin
  if not Open_Database(GetComponent(0)) then Exit;
  q := TsohoIBSQL.Create(nil);
  try
    q.Database := DBase;
    q.Transaction := DBase.DefaultTransaction;
    q.SQL.Add('select RDB$GENERATOR_NAME as GeneratorName from RDB$GENERATORS where (RDB$SYSTEM_FLAG is null)');
    q.SQL.Add('order by RDB$GENERATOR_NAME');
    q.ExecSQL;
    while not (Q.EOF) do begin
      Proc(Trim(q.Fields[0].AsString));
      q.Next;
    end; {--- while ---}
  finally
    q.Transaction.Commit;
    q.Close;
    q.Free;
  end; {--- try/finally ---}
end;

//-----------------------------------------------------------------------------
// TsohoFormIBFieldsProperty
//-----------------------------------------------------------------------------
function TsohoFormIBFieldsProperty.GetAttributes : TPropertyAttributes;
begin
  Result := [paValueList, paMultiSelect];
end;

procedure TsohoFormIBFieldsProperty.GetValues(Proc : TGetStrProc);
var FList : TStringList;
    i : Integer;
Begin
  if not Open_Database(GetComponent(0)) then Exit;
  if TName = '' then Exit;
  FList := CreateFieldsList(TName);
  try
    for I := 0 to FList.Count - 1 do Proc(FList.Strings[i]);
  finally
    DBase.DefaultTransaction.Commit;
    FList.Free;
  end; {--- try/finally ---}
End;

//-----------------------------------------------------------------------------
// TsohoFormIBGroupFieldsProperty
//-----------------------------------------------------------------------------
procedure TsohoFormIBGroupFieldsProperty.GetValues(Proc : TGetStrProc);
var FList : TStringList;
    i : Integer;
Begin
  if not Open_Database(GetComponent(0)) then Exit;
  if GName = '' then Exit;
  FList := CreateFieldsList(GName);
  try
    for I := 0 to FList.Count - 1 do Proc(FList.Strings[i]);
  finally
    DBase.DefaultTransaction.Commit;
    FList.Free;
  end; {--- try/finally ---}
End;

end.
