{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
{     TsohoIBRecordForm - ,  
       
}
unit SoIBRec;

{$I SOHOIBLIB.INC}

interface

uses SysUtils, WinTypes, WinProcs, Messages, Classes, Controls,
     SoCtmRec, SoIBClasses;

type

  { TsohoIBRecordForm - ,       
     .   TDataSet,    SQL
       .      ,
        SQL-    
    
  }
  TsohoIBRecordForm = class(TsohoCustomRecordForm)
  private
    FAutoCommit: boolean;
    FDataBase : TsohoIBDatabase;
    FTransaction : TsohoIBTransaction;
    Query : TsohoIBSQL;
  protected
    procedure Notification(aComponent : TComponent; aOperation : TOperation);override;
    function  RunUpdateSQL (SQLText : TStringList) : boolean;override;
    function  RunInsertSQL (SQLText : TStringList) : boolean;override;
    procedure PrepareCard; override;
    procedure SetMemoFields;
  public
    function GetIdentifier(const Value: string): string; override;
    constructor Create(aOwner: TComponent); override;
  published
    property AutoCommit: boolean read FAutoCommit write FAutoCommit default true;
    property DataBase : TsohoIBDatabase read FDataBase write FDataBase;
    property Transaction : TsohoIBTransaction read FTransaction write FTransaction;
  end;

implementation
uses SoAppUtils, SoIbRgst, SoDBCns, StdCtrls, DB, SoIBUtil, SoIBIdent,
     SoVCLUtils;

procedure TsohoIBRecordForm.PrepareCard;
begin
  if (FDataBase = nil) and (SingleIBRegister<>nil) then
    FDataBase := SingleIBRegister.DataBase;
  if (FTransaction = nil) and (SingleIBRegister<>nil) then
    FTransaction := SingleIBRegister.Transaction;
  inherited PrepareCard;
end;

procedure TsohoIBRecordForm.SetMemoFields;
var I, Index : LongInt;
    Strm : TMemoryStream;
    FieldControl : TWinControl;
begin
  I := 0;
  for Index := 0 to pred(Folder.DataSet.FieldCount) do
    if Folder.DataSet.Fields[Index] is TMemoField then begin
      Strm := TMemoryStream.Create;
      FieldControl := DialogForm.GetControlByIndex(index);
      TMemo(FieldControl).Lines.SaveToStream(Strm);
      Strm.Position := 0;
      Query.Params[I].LoadFromStream(Strm);
      inc(I);
      Strm.Free;
    end;
end;

function TsohoIBRecordForm.RunUpdateSQL (SQLText : TStringList) : boolean;
begin
  try
    Query := TsohoIBSQL.Create(Self);
    Query.DataBase := FDataBase;
    Query.Transaction := FTransaction;
    with Query do begin
     PushCursor(crHourGlass);
     SQL.Assign(SQLText);
     SetMemoFields;
     if FAutoCommit then NewTransactionStart(FTransaction);
     ExecQuery;
     if FAutoCommit then TransactionCommit(FTransaction);
     Result := True;
     DoAfterSQLExecute;
    end;
  except
   on E: Exception do begin
     Result := false;
     if FAutoCommit then TransactionRollback(FTransaction);
     ErrorMsg(Format(sohoRecFormUpdateError, [Self.name, E.Message]));
   end;
  end;
  Query.Free;
end;

function TsohoIBRecordForm.RunInsertSQL (SQLText : TStringList) : boolean;
begin
  try
    Query := TsohoIBSQL.Create(Self);
    Query.DataBase := FDataBase;
    Query.Transaction := FTransaction;
    with Query do begin
     PushCursor(crHourGlass);
     SQL.Assign(SQLText);
     SetMemoFields;
     if FAutoCommit then NewTransactionStart(FTransaction);
     ExecQuery;
     if FAutoCommit then TransactionCommit(FTransaction);
     Result := True;
     DoAfterSQLExecute;
    end;
  except
    on E:Exception do begin
     Result := false;
     if FAutoCommit then TransactionRollback(FTransaction);
     ErrorMsg(Format(sohoRecFormInsertError, [Self.name, E.Message]));
    end;
  end;
  Query.Free;
end;

procedure TsohoIBRecordForm.Notification(aComponent : TComponent; aOperation : TOperation);
begin
  if (aComponent = FTransaction) and (aOperation = opRemove) then
    FTransaction := nil;
  if (aComponent = FDataBase) and (aOperation = opRemove) then
    FDataBase := nil;
  inherited Notification(aComponent, aOperation);
end;

constructor TsohoIBRecordForm.Create(aOwner: TComponent);
begin
  inherited;
  FAutoCommit := true;
end;

function TsohoIBRecordForm.GetIdentifier(const Value: string): string;
begin
  if FDataBase = nil then
    result := inherited GetIdentifier(Value)
  else
    Result := FormatIdentifier(FDataBase.SQLDialect, Value);
end;

end.

