{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
{
     TsohoIBRegister.    
  SohoIBLib,     ,
    ,     
 .
}
unit SoIBRgst;

{$I SOHOIBLIB.INC}
{$I FIBPLUS.INC}

interface
uses Classes, SoCtmRgs, SoIBClasses, Forms;

type

  TsohoIBRegister = class(TsohoCustomRegister)
  private
    FDataBase : TsohoIBDatabase;
    FTransaction : TsohoIBTransaction;
    FOnDBConnect, FOnDBClose: TNotifyEvent;
    FHooked: boolean;
    FPassword: string;
    function GetUserName: string;
    procedure SetUserName(const Value: string);
  protected
    procedure Notification(aComponent : TComponent; aOperation : TOperation);override;
    procedure DatabaseConnect(Sender: TObject);
    procedure DatabaseClose(Sender: TObject);
    procedure SetEvents;
    procedure RestoreEvents;
    function ShowPasswordDialog: boolean; override;
    procedure GetUserRights; override;
  public
    procedure Loaded; override;
    constructor Create(aOwner : TComponent); override;
    destructor Destroy; override;
    procedure Activate; override;
  published
    property DataBase : TsohoIBDatabase read FDataBase write FDataBase;
    property Transaction : TsohoIBTransaction read FTransaction write FTransaction;
    property LoginUserName: string read GetUserName write SetUserName;
    property Password: string read FPassword write FPassword;
    {        }
    property ApplyRights;
    {        }
    property PasswordRequired default False;
    {       }
    property UserActionRights;
    {         }
    property UserDataReadRights;
    {         }
    property UserDataWriteRights;
    {    AppplyRights,     
              
         .      
        ,    . 
      HideSecret       ""
      :  Visible = false,  Enabled = false }
    property HideSecret default true;
    {  :   ,     }
    property ActivateOn;
    {     TsohoCustomRegister }
    property OnActivate;
    {     TsohoCustomRegister }
    property OnBeforeActivate;
    {  ,  TsohoCustomRegister   
         .    
      UserName       
      (PasswordRequired),   OnUserNameRequired   }
    property OnUserNameRequired;
  end;

  TsohoIBSubRegister = class(TsohoSubRegister)
  public
    function DataBase : TsohoIBDatabase;
    function Transaction : TsohoIBTransaction;
  end;

var
  SingleIBRegister : TsohoIBRegister;

implementation
uses SysUtils, SoUtils, SoDBCns, SoIBUtil, SoIBIdent, SoIBLgnDlg,
     SoStrUtils, SoUnit;

{ TsohoIBRegister }
constructor TsohoIBRegister.Create(aOwner : TComponent);
begin
  inherited Create(aOwner);
  SingleIBRegister := Self;
  FPassword := EmptyStr;
end;

procedure TsohoIBRegister.DatabaseClose(Sender: TObject);
begin
  if not (csDestroying in ComponentState) then ForgetDataBase;
  if Assigned(FOnDBClose) then FOnDBClose(Sender);
end;

procedure TsohoIBRegister.DatabaseConnect(Sender: TObject);
begin
  SetDatabase(FDatabase);
  if Assigned(FOnDBConnect) then FOnDBConnect(Sender);
end;

destructor TsohoIBRegister.Destroy;
begin
    SingleIBRegister := nil;
  if FHooked then begin
    FHooked := false;
    if Assigned(FDatabase) then begin
      FDatabase.OnConnect := FOnDBConnect;
      FDatabase.OnClose := FOnDBClose;
      if not (csDestroying in FDatabase.ComponentState) then FDatabase.Close;
    end;     
  end;
  inherited Destroy;
end;

procedure TsohoIBRegister.Loaded;
begin
  inherited;
  SetEvents;
end;

procedure TsohoIBRegister.Notification(aComponent : TComponent; aOperation : TOperation);
begin
  if (aComponent = FTransaction) and (aOperation = opRemove) then
    FTransaction := nil;
  if (aComponent = FDataBase) and (aOperation = opRemove) then begin
    FDataBase := nil;
  end;
  inherited Notification(aComponent, aOperation);
end;

procedure TsohoIBRegister.Activate;
begin
  if (PathToData <> EmptyStr) and Assigned(FDatabase) then
    FDatabase.DBName := PathToData;
  inherited;  
end;

function TsohoIBRegister.ShowPasswordDialog: boolean;
begin
  Result := false;
  if not Assigned(FDatabase) then exit;

  Result := ShowIBLoginDialog(ShortName, FPassword, FDatabase);

  if Result then begin
    if LoginUserName <> EmptyStr then FDatabase.UserName := LoginUserName;
    if Password <> EmptyStr then FDatabase.Password := Password;
  end;  
end;

procedure TsohoIBRegister.GetUserRights;
var TmpTr: TsohoIBTransaction;
    TmpQ: TsohoIBSQL;
begin
  DataReadRights := [0..255];
  DataWriteRights := [0..255];
  ActionRights := [0..255];
  UserID := 0;
  if not Assigned(FDatabase) then exit;

  if not Database.Connected then
    try
      Database.UserName := LoginUserName;
      Database.Password := Password;
      Database.Open;
    except
      exit;
    end;
    
  //     
  if Database.TableExists('SOHO_PROTECT') then begin
    TmpTr := TsohoIBTransaction.Create(Self);
    TmpTr.DefaultDatabase := FDatabase;

    TmpQ := TsohoIBSQL.Create(Self);
    with TmpQ, SQL do begin
      Database := FDatabase;
      Transaction := TmpTr;
      TmpTr.StartTransaction;

      Add('select * from SOHO_PROTECT where USER_NAME = ' +
        DoubleQuotes(AnsiUpperCase(LoginUserName)));
      try
        ExecSQL;
        {$IFNDEF FIELD_BY_NAME_AS_FUNC}
        if FieldByName['USER_NAME'].AsString <> EmptyStr then begin
          DataReadRights := StrToSet(FieldByName['READ_RIGHTS'].AsString);
          DataWriteRights := StrToSet(FieldByName['WRITE_RIGHTS'].AsString);
          ActionRights := StrToSet(FieldByName['ACTION_RIGHTS'].AsString);
          UserID := FieldByName['ID'].AsInteger;
          UserName := LoginUserName;
        end
        {$ELSE}
        if FieldByName('USER_NAME').AsString <> EmptyStr then begin
          DataReadRights := StrToSet(FieldByName('READ_RIGHTS').AsString);
          DataWriteRights := StrToSet(FieldByName('WRITE_RIGHTS').AsString);
          ActionRights := StrToSet(FieldByName('ACTION_RIGHTS').AsString);
          UserID := FieldByName('ID').AsInteger;
          UserName := LoginUserName;
        end
        {$ENDIF}
        else begin
          DataReadRights := [0..0];
          DataWriteRights := [0..0];
          ActionRights := [0..0];
          UserID := 0;
          UserName := EmptyStr;
        end;
      except
        DataReadRights := [0..0];
        DataWriteRights := [0..0];
        ActionRights := [0..0];
        UserID := 0;
        UserName := EmptyStr;
      end;
      TmpTr.Commit;
    end;
    TmpQ.Free;
    TmpTr.Free;
  end;
end;

function TsohoIBRegister.GetUserName: string;
begin
  Result := ShortName;
end;

procedure TsohoIBRegister.SetUserName(const Value: string);
begin
  ShortName := Value;
end;

{ TsohoIBSubRegister }
function TsohoIBSubRegister.DataBase : TsohoIBDatabase;
begin
  Result := nil;
  if SingleIBRegister = nil then exit;
  Result := SingleIBRegister.DataBase;
end;

function TsohoIBSubRegister.Transaction : TsohoIBTransaction;
begin
  Result := nil;
  if SingleIBRegister = nil then exit;
  Result := SingleIBRegister.Transaction;
end;

procedure TsohoIBRegister.RestoreEvents;
begin
  if FHooked and Assigned(FDatabase) then begin
    FDatabase.OnConnect := FOnDBConnect;
    FDatabase.OnClose := FOnDBClose;
  end;
  FHooked := false;
end;

procedure TsohoIBRegister.SetEvents;
begin
  if Assigned(FDatabase) then begin
    FHooked := true;
    FOnDBConnect := FDatabase.OnConnect;
    FOnDBClose := FDatabase.OnClose;
    FDatabase.OnConnect := DatabaseConnect;
    FDatabase.OnClose := DatabaseClose;
  end;
end;

initialization
  SingleIBRegister := nil;
end.
