{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
unit SoIBSQLMonitor;

{$I SOHOIBLIB.INC}
interface

uses Windows, Messages, SysUtils, Classes, FIBSQLMonitor;


type

  TsohoTraceOption = (
      toConnect,
      toDisconnect,
      toTransactionStart,
      toCommit,
      toCommitRetaining,
      toRollback,
      toSQLPrepare,
      toSQLExecute,
      toSQLFetch,
      toSingleApplication
  );

  TsohoTraceOptions = set of TsohoTraceOption;

  TsohoTraceEvent = procedure (Sender: TObject;
     const ApplicationName: string;
     const SQLObjectName: string;
     TraceOperation: TsohoTraceOption;
     const SQLText: string) of object;

  TsohoTraceNewApplication = procedure (Sender: TObject;
     const ApplicationName: string; var AddToTraceList : boolean) of object;

const

  DefTraceOptions = [
     toConnect, toDisconnect, toTransactionStart,
     toCommit, toCommitRetaining, toRollback,
     toSQLPrepare, toSQLExecute, toSQLFetch
  ];

type

  TsohoObjectsList = class(TStringList)
  public
    procedure Insert(Index: Integer; const S: string); override;
  end;

  TsohoIBSQLMonitor = class(TFIBCustomSQLMonitor)
  private
    { Private declarations }
    FTraceOptions: TsohoTraceOptions;
    FTraceObjects: TsohoObjectsList;
    FTraceApplications: TsohoObjectsList;
    FCurrentApplication: string;
    FOnTrace: TsohoTraceEvent;
    FOnNewApplication: TsohoTraceNewApplication;
  protected
    { Protected declarations }
    procedure SQLEventHandler(EventText: String; EventTime : TDateTime); virtual;
    function GetTraceObjects: TStrings;
    function GetTraceApplications: TStrings;
    procedure SetTraceObjects(Value: TStrings);
    procedure SetTraceApplications(Value: TStrings);
    function ApplicationRegistered(const AppName: string): boolean;
    function ObjectRegistered(const ObjName: string): boolean;
  public
    { Public declarations }
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property TraceOptions: TsohoTraceOptions read FTraceOptions write FTraceOptions default DefTraceOptions;
    property SQLObjects: TStrings read GetTraceObjects write SetTraceObjects;
    property TraceApplications: TStrings read GetTraceApplications write SetTraceApplications;
    property TraceApplication: string read FCurrentApplication write FCurrentApplication;
    property OnTrace: TsohoTraceEvent read FOnTrace write FOnTrace;
    property OnNewApplication: TsohoTraceNewApplication read FOnNewApplication write FOnNewApplication;
  end;

implementation

{ TsohoObjectsList }
procedure TsohoObjectsList.Insert(Index: Integer; const S: string);
begin
  inherited Insert(Index, AnsiUpperCase(S));
end;

{ TsohoIBSQLMonitor }
function TsohoIBSQLMonitor.ApplicationRegistered(const AppName: string): boolean;
var Index: LongInt;
begin
  Result := true;
  for Index := 0 to pred(FTraceApplications.Count) do
    if AnsiUpperCase(FTraceApplications[Index]) = AppName then exit;
  Result := false;
end;

function TsohoIBSQLMonitor.ObjectRegistered(const ObjName: string): boolean;
var Index: LongInt;
begin
  Result := true;
  for Index := 0 to pred(FTraceObjects.Count) do
    if AnsiUpperCase(FTraceObjects[Index]) = ObjName then exit;
  Result := false;
end;

function TsohoIBSQLMonitor.GetTraceObjects: TStrings;
begin
  Result := FTraceObjects;
end;

function TsohoIBSQLMonitor.GetTraceApplications: TStrings;
begin
  Result := FTraceApplications;
end;

procedure TsohoIBSQLMonitor.SetTraceObjects(Value: TStrings);
begin
  FTraceObjects.Assign(Value);
end;

procedure TsohoIBSQLMonitor.SetTraceApplications(Value: TStrings);
begin
  FTraceApplications.Assign(Value);
end;

const
   sqlApplication = '[Application: ';

   sqlOperations : array [TsohoTraceOption] of string = (
      'Connect]',
      'Disconnect]',
      'Start transaction]',
      'Commit (Hard commit)]',
      'Commit retaining (Soft commit)]',
      'Rollback]',
      'Prepare] ',
      'Execute] ',
      'Fetch] ',
      ''
   );

procedure TsohoIBSQLMonitor.SQLEventHandler(EventText: String; EventTime : TDateTime);
var AName,OName: string;
    P, Fp, ToP{, AppIndex}: LongInt;
    Index: TsohoTraceOption;
    AddToList: boolean;
begin
  if not Assigned(FOnTrace) then exit;
  //     :)

  //    -
  P := Pos(sqlApplication, EventText);
  if P <> 0 then begin
    Fp := P + length(sqlApplication);
    ToP := pos(']', EventText);
    aName := copy(EventText, Fp, ToP - Fp);
    EventText := copy (EventText, ToP + 5, length(EventText)); //   CRLF
  end;
  //  ,      ?
  // AppIndex := FTraceApplications.IndexOf(AnsiUpperCase(aName));
  AddToList := (not ApplicationRegistered(AnsiUpperCase(aName)))
    and Assigned(FOnNewApplication);
  if AddToList then begin
    FTraceApplications.Add(AnsiUpperCase(aName));
    FOnNewApplication(Self, aName, AddToList);
  end;

  if AddToList then FTraceApplications.Add(AnsiUpperCase(aName));

  if (FTraceApplications.Count > 0) and
     (not ApplicationRegistered(AnsiUpperCase(aName))) then exit;

  if (toSingleApplication in TraceOptions) and
     (CompareText(TraceApplication, aName) <> 0) then exit;

  //    
  OName := '';
  P := Pos(': [', EventText);
  if P <> 0 then begin
    OName := copy(EventText, 1, P - 1);
    EventText := copy(EventText, P + 1, length(EventText));
  end;

  // -, ,      ?
  if (OName <> '') and
     (FTraceObjects.Count > 0) and
     (not ObjectRegistered(AnsiUpperCase(OName))) then exit;

  //   
  for Index := toConnect to toSQLFetch do begin
    P := Pos(sqlOperations[Index], EventText);
    if (P <> 0) and (Index in FTraceOptions) then begin
      Fp := P + length(sqlOperations[Index]);
      EventText := trim(copy(EventText, Fp, length(EventText)));
      FOnTrace(Self, aName, oName, Index, EventText);
      exit;
    end;
  end;
end;

constructor TsohoIBSQLMonitor.Create(aOwner: TComponent);
begin
  inherited;
  FTraceOptions := DefTraceOptions;
  FTraceObjects := TsohoObjectsList.Create;
  FTraceApplications := TsohoObjectsList.Create;
  FCurrentApplication := '';
  OnSQL := SQLEventHandler;
end;

destructor TsohoIBSQLMonitor.Destroy;
begin
  OnSQL := nil;
  FTraceObjects.Free;
  FTraceApplications.Free;
  inherited;
end;

end.
