{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
unit SoIBTVCont;

{$I SOHOIBLIB.INC}

interface
uses Classes, SoCtmTVCont, SoIBClasses, SoIBRec, SoIBFolder,
     SoIBTreeView, SoIBDBGridAddon;

type

  TsohoIBTreeViewForm = class(TsohoCustomTreeViewForm)
  private
    FRecTr, FTransaction: TsohoIBTransaction;
  public
    constructor Create(aOwner: TComponent); override;
    property RecordTransaction: TsohoIBTransaction read FRecTr;
    property Transaction: TsohoIBTransaction read FTransaction;
  end;

  TsohoIBTVContainer = class(TsohoCustomTVContainer)
  private
    FDatabase: TsohoIBDatabase;
  protected
    function CreateTreeViewForm: TsohoCustomTreeViewForm; override;
    procedure SetTreeViewProperties; override;
    procedure SetFolderProperties; override;
    procedure SetRecordProperties; override;
    procedure SetQueryProperties; override;
    procedure SetEvents; override;
  published
    property Database: TsohoIBDatabase read FDatabase write FDatabase;
  end;

implementation
uses SoIBRgst, Forms, Controls;

{ TsohoIBTreeViewForm }

constructor TsohoIBTreeViewForm.Create(aOwner: TComponent);
begin
  inherited;
  FTransaction := TsohoIBTransaction.Create(Self);
  FRecTr := TsohoIBTransaction.Create(Self);

  FTreeView := TsohoIBTreeView.Create(Self);
  TsohoIBTreeView(FTreeView).Transaction := FTransaction;

  FRecordForm := TsohoIBRecordForm.Create(Self);
  TsohoIBRecordForm(FRecordForm).Transaction := FRecTr;

  FFolder := TsohoIBFolder.Create(Self);
  TsohoIBFolder(FFolder).Transaction := FTransaction;

  FGroupsDS := TsohoIBDataSet.Create(Self);
  TsohoIBDataSet(FGroupsDS).Transaction := FTransaction;

  FItemsDS := TsohoIBDataSet.Create(Self);
  TsohoIBDataSet(FItemsDS).Transaction := FTransaction;

  FDBAddon := TsohoIBDBGridAddon.Create(Self);
end;

{ TsohoIBTVContainer }

function TsohoIBTVContainer.CreateTreeViewForm: TsohoCustomTreeViewForm;
begin
  Result := TsohoIBTreeViewForm.Create(Application);
  Result.TreeView.Parent := Result;
  Result.TreeView.Align := alClient;
end;

procedure TsohoIBTVContainer.SetEvents;
begin
  if TreeViewForm = nil then exit;
  if (FDataBase = nil) and (SingleIBRegister <> nil) then
     FDataBase := SingleIBRegister.DataBase;
  with TsohoIBTreeViewForm(TreeViewForm) do begin
    FTransaction.DefaultDatabase := FDataBase;
    FRecTr.DefaultDatabase := FDataBase;
  end;
  inherited;
end;

procedure TsohoIBTVContainer.SetFolderProperties;
begin
  inherited;
  with TreeViewForm do begin
    TsohoIBFolder(Folder).DataBase := FDataBase;
    Folder.Auto := false;
  end
end;

procedure TsohoIBTVContainer.SetQueryProperties;
var FGrpsSQL, FItemsSQL : string;
begin
  inherited;
  with TreeViewForm do begin
    FGrpsSQL  := GroupsSQLFile;
    FItemsSQL := ItemsSQLFile;

    if SingleIBRegister <> nil then begin
      FGrpsSQL  := SingleIBRegister.PathToSQL + GroupsSQLFile;
      FItemsSQL := SingleIBRegister.PathToSQL + ItemsSQLFile;
    end;

    with TsohoIBDataSet(GroupsDS), SelectSQL do begin
      DataBase := FDataBase;
      LoadFromFile(FGrpsSQL);
    end;
    with TsohoIBDataSet(ItemsDS), SelectSQL do begin
      DataBase := FDataBase;
      LoadFromFile(FItemsSQL);
    end;
  end;
end;

procedure TsohoIBTVContainer.SetRecordProperties;
begin
  inherited;
  with TreeViewForm do begin
    TsohoIBRecordForm(RecordForm).DataBase := FDataBase;
    TsohoIBRecordForm(RecordForm).Folder := Folder;
  end;
end;

procedure TsohoIBTVContainer.SetTreeViewProperties;
begin
  inherited;
  with TreeViewForm, TsohoIBTreeView(TreeView) do begin
    DataBase := FDataBase;
    ItemsDataSet := ItemsDS;
    GroupsDataSet := GroupsDS;
  end;
  with TreeViewForm do
    TsohoIBTreeView(TreeView).Folder := Folder;
end;

end.
