{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
unit SoIBTreeView;

{$I SOHOIBLIB.INC}

interface
uses SysUtils, WinTypes, WinProcs, Messages, Classes, Controls, DB,
     SoIBClasses, SoCtmTre, SoDBGrid, ComCtrls;

type

  {  TsohoIBTreeView -  ,    :
    TsohoEnhTreeView ( GroupTreView)  TsohoDBGrid ( DBGrid).  
         -     .
    ,       .   
      Id (ftInteger) -   , Hash (ftInteger) - 
        , IdGrp (ftInteger) -   
    ,     , Name -  .    
          TsohoIBTreeView.     
    ,       OrderField.   
      : Id, Hash, IdParent (ftInteger) -   
     , Name -  .     ,  
      .      design-time,  
    Component Editor',   TsohoIBTreeView.   TsohoIBTreeView 
     TDataSet',         ,   -  
    .       GroupsDataSet  ItemsDataSet.  ,
           GroupsTable  ItemsTable (,
    ItemsTable := Items).     TsohoIBTreeView 
     ,         
     OnNewItem  OnItemEdit.     
          TsohoTVContainer.
     :  "" -      OutLine - 
       OutLine!     ,    
     'Id'      !  ,  
    TsohoIBTreeView,      .  ,
         ,     
     Hash,  TsohoIBTreeView     :
        Drag&Drop   , 
        ,   .  Ctrl+Ins 
    Shift-Ins   ""   .   
        ,      
          . TsohoIBTreeView  
       ( Folder)      ,
       GroupTreeView,    ,    
    IncludeSubTree.
  }
  TsohoIBTreeView = class(TsohoCustomTreeView)
  private
    { Private declarations }
    FDefAnswer: integer; // 0 - yestoall, 1 - notoall, 2 - not defined
    FCanMany: boolean; //      
    OperationQuery: TsohoIBSQL;
    FDataBase : TsohoIBDatabase;
    FTransaction : TsohoIBTransaction;
  protected
    { Protected declarations }
    procedure BeforeActivate; override;
    procedure InternalRefreshFilter; override;
    function CreateGroupsTree: boolean;override;
    function  RenumberRecords: boolean;override;
    function  DeleteOneRecord(ItemID: Longint): boolean;override;
    procedure MoveGroup (NewParentId, NewHash, Id : LongInt);override;
    function  TreeViewGetGroupsIds(const aFixedGrpName, aGroupTable: string;
              var Ids: TsohoDBGridResults): boolean;override;
    procedure Notification(aComponent : TComponent; aOperation : TOperation);override;
  public
    {          }
    function QuietAddGroup(var NewName: string): boolean; override;
    {          }
    function QuietEditGroup(var NewName: string): boolean; override;
    {   }
    function DeleteGroup: boolean; override;
    {   }
    function MoveItem: boolean; override;
    {   }
    function DeleteItem: boolean; override;
    {     Hash    }
    function GetNewItemHash(GrpId: Longint): Longint;override;
    {     .   
      Id, IdGrp, Hash }
    function AddNewItem(GrpId, ItemID, ItemHash: Longint): boolean; override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DataBase : TsohoIBDatabase read FDataBase write FDataBase;
    property Transaction : TsohoIBTransaction read FTransaction write FTransaction;
  end;

function GroupsIdsToStr(const aKeyField, GrpField, aFixedGrpName, GroupTable : string;
  aDataBase: TsohoIBDataBase; aTransaction : TsohoIBTransaction): string;

implementation
uses SohoWrds, SoAppUtils, SoUnit, SoIBRgst, SoDBCns, SoIBUtil,
     SoIBIdent, SoIBCns, SoDBRtn, Forms, SoVCLUtils, SoStrUtils,
     SoDialogs;

function GetGroupsIds(const aKeyField, aFixedGrpName, aGroupTable : string;
   aDataBase: TsohoIBDataBase; aTransaction : TsohoIBTransaction;
   var Ids: TsohoDBGridResults): boolean;
var Query: TsohoIBDataSet;
  AddedCount      : Integer;
  CurId, CurParent: Longint;
begin
  Result := False;
  Query := TsohoIBDataSet.Create(nil);
  PushCursor(crHourGlass);
  with Query, SelectSQL do begin
    DataBase := aDataBase;
    Transaction := aTransaction;
    Add('select * from ' +
       FormatIdentifier(aDataBase.SQLDialect, aGroupTable));
    Add('order by IdParent, ' +
       FormatIdentifier(aDataBase.SQLDialect, aKeyField));
    try
      Prepare;
      Open;
    except
      aTransaction.Rollback;
      Free;
      exit;
    end;
  end;
  with Query do begin
    First;
    while (not EOF) and (FieldByName('Name').AsString <> aFixedGrpName) do Next;
    if FieldByName('Name').AsString = aFixedGrpName then
      Ids.Add(FieldByName(aKeyField).AsInteger)
    else begin
      aTransaction.Rollback;
      Query.Free;
      RestoreCursor;
      exit;
    end;
    AddedCount := -1;
    while AddedCount <> 0 do begin
      First;
      AddedCount := 0;
      while not EOF do begin
        CurId := FieldByName(aKeyField).AsInteger;
        CurParent := FieldByName('IdParent').AsInteger;
        if (Ids.IndexOf(CurParent) <> -1) and
          (Ids.IndexOf(CurId) = -1) then begin
          Ids.Add(CurId);
          Inc(AddedCount);
        end;
        Next;
      end;
    end;
    aTransaction.Rollback;
    Free;
  end;
  Result := True;
  RestoreCursor;
end;

function GroupsIdsToStr(const aKeyField, GrpField, aFixedGrpName, GroupTable : string;
  aDataBase: TsohoIBDataBase; aTransaction : TsohoIBTransaction): string;
var Ids: TsohoDBGridResults;
  index: Longint;
begin
  Ids := TsohoDBGridResults.Create;
  Result := '(' + FormatIdentifier(aDataBase.SQLDialect, GrpField) + ' = 0)';
  if GetGroupsIds(aKeyField, aFixedGrpName, GroupTable, aDataBase, aTransaction, Ids) then
    for index := 0 to pred(Ids.Count) do begin
      Result := Result + '(' +
        FormatIdentifier(aDataBase.SQLDialect, GrpField) + '=' +
        IntToStr(Ids.Items[index]) + ')';
      if index <> pred(Ids.Count) then Result := Result + ' or ';
    end;
  Ids.Free;
end;

{ TsohoIBTreeView }
procedure TsohoIBTreeView.BeforeActivate;
begin
  inherited;
  TransactionCommit(FTransaction);
  if not NewTransactionStart(FTransaction) then begin
    ErrorMsg(sohoIBTreeViewUnableStartTransaction);
    exit;
  end;
end;

procedure TsohoIBTreeView.InternalRefreshFilter;
begin
  try
    // statements to try
    TransactionCommit(FTransaction);
    if not NewTransactionStart(FTransaction) then begin
      ErrorMsg(sohoIBTreeViewUnableStartTransaction);
      exit;
    end;
    if Assigned(Folder) then RefreshFolderWithId(Folder, KeyField)
    else RefreshDataSet(ItemsDataSet);
  except
    on e: Exception do begin
      Application.HandleException(Self);
      ErrorMsg(E.Message);
    end;
  end;    // try/except
end;

function  TsohoIBTreeView.TreeViewGetGroupsIds(const aFixedGrpName, aGroupTable: string;
              var Ids: TsohoDBGridResults): boolean;
begin
  Result := GetGroupsIds(KeyField, aFixedGrpName, aGroupTable,
    FDataBase, FTransaction, Ids);
end;

function TsohoIBTreeView.QuietAddGroup(var NewName: string): boolean;
var LastChildHash, NewGroupHash,
  NewGroupId, ParentId: Longint;
  Query               : TsohoIBSQL;
  Allow               : boolean;
  F1, F2, F3          : string;
  aOnNewCust          : TsohoTreeCustomGroup;
  aOnNew              : TsohoTreeOnNewGroup;
  TmpLeaf             : PTreeLeaf;
  FieldsList          : TStringList;
  ValuesList          : TStringList;
  ParentNode, LastChild: TTreeNode;

  procedure CreateInsertQuery;
  Var
    i : Integer;
  begin
    Query := TsohoIBSQL.Create(Self);
    Query.DataBase := FDataBase;
    Query.Transaction := FTransaction;
    with Query, SQL do begin
      Add('Insert into ' + FormatIdentifier(FDataBase.SQLDialect, GroupsTable));
      Add('(' +
         FormatIdentifier(FDataBase.SQLDialect, KeyField) + ', ' +
         FormatIdentifier(FDataBase.SQLDialect, HashField) + ', ' +
         FormatIdentifier(FDataBase.SQLDialect, NameParentID) + ', Name');
      if FieldsList <> nil then
        for i := 0 to pred(FieldsList.Count) do Add(', '+FieldsList[i]);
      Add(')');
      NewGroupId := SoUnit.GetNewId(GetNewGroupIdPreffix);
      Add('Values (' + IntToStr(NewGroupId) + ', ' + IntToStr(NewGroupHash) + ',');
      ParentId := GetIdByNode(ParentNode);
      Add(IntToStr(ParentId) + ', ' + SingleQuotes(NewName) );
      if ValuesList <> nil then
        for i := 0 to pred(ValuesList.Count) do Add(', '+ValuesList[i]);
      Add(')');
    end;
  end;

begin
  Result := False;
  FieldsList := nil;
  ValuesList := nil;
  WordToWordForm(GroupsName, F1, F2, F3, GroupsGenderIndex);
  with GroupTreeView do begin
    ParentNode := Selected;
    LastChild := ParentNode.GetLastChild;
    if LastChild = nil then NewGroupHash := 0
    else begin
      TmpLeaf := GetLeafRecord(LastChild);
      if Assigned(TmpLeaf) then LastChildHash := TmpLeaf^.ItemHash
      else LastChildHash := 0;
      NewGroupHash := GetHash(LastChildHash, MaxLongint);
    end;
  end;
  {  Hash    }
  try
    CreateInsertQuery;
    try
      Allow := True;
      aOnNewCust := OnCustomNewGrp;
      aOnNew     := OnNewGrp;

      if Assigned(aOnNewCust) then aOnNewCust(Self, NewGroupId,
        NewGroupHash, ParentId, NewName, Allow)
      else
        if Assigned(aOnNew) then begin
          FieldsList := TStringList.Create;
          ValuesList := TStringList.Create;
          aOnNew(Self, SohoTreeNode(NewName, NewGroupId), FieldsList, ValuesList, Allow);
          CreateInsertQuery;
          FieldsList.Free;
          ValuesList.Free;
        end; {--- if ---}
      if Allow then begin
        if not Assigned(aOnNewCust) then begin
          PushCursor(crHourGlass);
          TransactionStart(FTransaction);
          Query.Prepare;
          Query.ExecQuery;
          TransactionCommit(FTransaction);
          RefreshItems;
        end;
        {     }
        if AddTreeItem(ParentNode, NewGroupId, ParentId, NewGroupHash,
          NewName) then
          ParentNode.Expand(false);
          with GroupTreeView do begin
            Selected := GetNodeByName(NewName);
          end;
        {   OutLine }
        Result := True;
      end;
      if Allow and Assigned(aOnNewCust) then RefreshTreeView;
      Changed;
    except
      // RestoreCursor;
      TransactionRollback(FTransaction);
      ErrorMsg(Format(sohoTreeGroupDontCreate, [Name, F1]));
    end;
  finally
    Query.Free;
    RestoreCursor;
  end;
end;

function TsohoIBTreeView.MoveItem: boolean;
var Allow: boolean;
  index: Integer;

  procedure DoMoveItem(aItemId, aNewGrpId, aNewHash: Longint);
  var aOnMove : TsohoTreeOnMoveItem;
  begin
    OperationQuery.DataBase := FDataBase;
    OperationQuery.Transaction := FTransaction;
    with OperationQuery, OperationQuery.SQL do begin
      Allow := True;
      aOnMove := OnItemMove;
      if Assigned(aOnMove) then
        aOnMove(Self, FMoveItem.OldGrpId, aNewGrpId, aItemId, aNewHash, Allow);
      if not Allow then exit;
      Clear;
      Add('update ' + FormatIdentifier(FDataBase.SQLDialect, ItemsTable));
      Add('set ' +
          FormatIdentifier(FDataBase.SQLDialect, NameGroupID) + '=' + IntToStr(aNewGrpId) + ', ' +
          FormatIdentifier(FDataBase.SQLDialect, HashField) + '=' + IntToStr(aNewHash));
      Add('where ' +
          FormatIdentifier(FDataBase.SQLDialect, KeyField) + '=' + IntToStr(aItemId));
      try
        try
          PushCursor(crHourGlass);
          TransactionStart(FTransaction);
          Prepare;
          ExecQuery;
          TransactionCommit(FTransaction);
          RefreshItems;
          Result := True;
          Changed;
        except
          TransactionRollback(FTransaction);
          ErrorMsg(Format(sohoTreeErrorOnItemMove, [Name]));
        end;
      finally
        RestoreCursor;
      end;
    end;
  end;

begin
  { Event on drag and drop }
  Result := False;
  if not (trMoveItem in Options) then exit;
  if ItemsTable = '' then begin
    ErrorMsg(Format(sohoTreeItemsTableNotDefined,[Name]));
    exit;
  end;
  if DBGrid.Results.Count = 0 then DoMoveItem(FMoveItem.ID, FMoveItem.NewGrpId,
    FMoveItem.NewHash)
  else
    for index := 0 to pred(DBGrid.Results.Count) do begin
      FMoveItem.NewHash := GetHash(FMoveItem.NewHash, RightHash);
      DoMoveItem(DBGrid.Results.Items[index], FMoveItem.NewGrpId,
                 FMoveItem.NewHash);
    end;
  DBGrid.Results.Clear;
  RefreshItems;
end;

function TsohoIBTreeView.QuietEditGroup(var NewName: string): boolean;
var 
  aGroupId, aGroupHash,
   aParentId: Longint;
  Query      : TsohoIBSQL;
  Allow      : boolean;
  CurNode: TTreeNode;

  procedure CreateUpdateQuery;
  var TmpLeaf: PTreeLeaf;
  begin
    Query := TsohoIBSQL.Create(Self);
    Query.DataBase := FDataBase;
    Query.Transaction := FTransaction;
    with Query, SQL do begin
      Add('Update ' + FormatIdentifier(FDataBase.SQLDialect, GroupsTable));
      Add('Set Name = ' + SingleQuotes(NewName));
      TmpLeaf := GetLeafRecord(CurNode);
      if Assigned(TmpLeaf) then begin
        aGroupId := TmpLeaf^.ItemID;
        aGroupHash := TmpLeaf^.ItemHash;
        aParentId := TmpLeaf^.ParentId;
      end
      else begin
        aGroupId := 0;
        aGroupHash := 0;
        aParentId := 0;
      end;
      Add('Where ' + FormatIdentifier(FDataBase.SQLDialect, KeyField) + '=' + IntToStr(aGroupId));
    end;
  end;

var aOnCustEd : TsohoTreeCustomGroup;
    aOnEdit   : TsohoTreeOnEditGroup;
begin
  Result := False;
  CurNode := GroupTreeView.Selected;
  try
    try
      CreateUpdateQuery;
      Allow := True;
      aOnCustEd := OnCustomEditGrp;
      aOnEdit := OnGrpEdit;
      if Assigned(aOnCustEd) then aOnCustEd(Self, aGroupId,
        aGroupHash, aParentId, NewName, Allow)
      else if Assigned(aOnEdit) then aOnEdit(Self,
        SohoTreeNode(CurNode.Text, aGroupId), Allow);
      if Allow then begin
        if not Assigned(aOnCustEd) then begin
          PushCursor(crHourGlass);
          TransactionStart(FTransaction);
          Query.Prepare;
          Query.ExecQuery;
          TransactionCommit(FTransaction);
          RefreshItems;
        end;
        CurNode.Text := NewName;
        Result := True;
        if Allow and Assigned(aOnCustEd) then RefreshTreeView;
        Changed;
      end;
    except
      TransactionRollback(FTransaction);
      ErrorMsg(Format(sohoTreeGroupRenameError, [Name]));
    end;
  finally
    Query.Free;
    RestoreCursor;
  end;
end;

function TsohoIBTreeView.DeleteGroup: boolean;
var JustOneDelete: boolean;
    WasChecked: boolean;

  function DeleteSubTree(Node: TTreeNode; GrpId: Longint): boolean;
  var GrpName: string;
    TopNode, CurNode: TTreeNode;
    DeleteThatGroup      : boolean;
    aOnDel               : TsohoTreeOnDeleteGroup;
    Confirm: TModalResult;

  begin
    GrpName := Node.Text;
    aOnDel := OnGrpDelete;
    case FDefAnswer of
      0: Result := true;
      1: begin
        Result := false;
        exit;
      end;
      else begin
        if Assigned(aOnDel) then aOnDel(Self, GrpId, Result)
        else begin
          if Node.Count > 0 then begin
            Confirm := MessageDlg(Format(sohoTreeDeleteGroup, [GrpName]),
               mtConfirmation, [mbYes, mbYesToAll, mbNo, mbNoToAll], 0);
            if Confirm = mrYesToAll then FDefAnswer := 0;
            if Confirm = mrNoToAll then FDefAnswer := 1;
            Result := Confirm in [mrYes, mrYesToAll];
          end
          else Result := YesNoMsg(Format(sohoTreeDeleteGroup, [GrpName]));
        end;
        if not Result then exit;
      end;
    end;
    TopNode := Node.GetFirstChild;
    {   ,  ...  :( }
    {  ,  ,        
          ,     . ,    
             :) }
    DeleteThatGroup := True;
    if Assigned(TopNode) then begin
      CurNode := TopNode;
      while CurNode <> nil do begin
        if not DeleteSubTree(CurNode, GetIdByNode(CurNode)) then
          DeleteThatGroup := False
        else JustOneDelete := True;
        CurNode := Node.GetNextChild(CurNode);
      end;
    end;
    {     -   ,
           IdGrp  }
    try
      ItemsDataSet.DisableControls;
      GroupTreeView.Selected := Node;
      OperationQuery.Database := FDataBase;
      OperationQuery.Transaction := FTransaction;
      with OperationQuery, OperationQuery.SQL do begin
        Clear;
        Add('delete from ' + FormatIdentifier(FDataBase.SQLDialect, ItemsTable));
        Add('Where ' + FormatIdentifier(FDataBase.SQLDialect, KeyField) + '=?Id');
      end;
      FCanMany := ItemsDataSet.RecordCount > 1;
      with ItemsDataSet do begin
        while not EOF do begin
          if not DeleteOneRecord(ItemsDataSet.FieldByName(KeyField).AsInteger)
            then DeleteThatGroup := False
          else JustOneDelete := True;
          Next;
        end;
      end;
      {       }
      if DeleteThatGroup then
        with OperationQuery, OperationQuery.SQL do begin
          Clear;
          Add('delete from ' + FormatIdentifier(FDataBase.SQLDialect, GroupsTable));
          Add('Where ' + FormatIdentifier(FDataBase.SQLDialect, KeyField) + '=' + IntToStr(GrpId));
          try
            Prepare;
            ExecQuery;
            JustOneDelete := True;
          except
            ErrorMsg(Format(sohoTreeGroupDeleteError, [Name, GrpName]));
          end;
        end;
    finally
      ItemsDataSet.EnableControls;
      Result := DeleteThatGroup;
    end;
  end;

var aAfterDel : TNotifyEvent;
begin
  FDefAnswer := 2;
  Result := False;
  JustOneDelete := False;
  if not (trDeleteGroup in Options) then exit;
  WasChecked := ChildCheckBox.Checked;
  ChildCheckBox.Checked := False;
  TransactionStart(FTransaction);
  Result := DeleteSubTree(GroupTreeView.Selected,
    GetIdByNode(GroupTreeView.Selected));
  if Result then TransactionCommit(FTransaction)
  else TransactionRollback(FTransaction);

  {     ,   }
  ChildCheckBox.Checked := WasChecked;
  if not JustOneDelete then begin
    RefreshItems;
    exit;
  end;
  aAfterDel := AfterGrpDelete;
  if Assigned(aAfterDel) then aAfterDel(Self);
  {    -   !}
  TreeViewButtonsClick(actRefresh);
  Changed;
  RefreshItems;
end;

function TsohoIBTreeView.AddNewItem(GrpId, ItemID, ItemHash: Longint): boolean;
begin
  PushCursor(crHourGlass);
  Result := False;
  OperationQuery.DataBase := FDataBase;
  OperationQuery.Transaction := FTransaction;
  with OperationQuery, OperationQuery.SQL do begin
    Clear;
    Add('insert into ' +
      FormatIdentifier(FDataBase.SQLDialect, ItemsTable) + ' (' +
      FormatIdentifier(FDataBase.SQLDialect, KeyField) + ', ' +
      FormatIdentifier(FDataBase.SQLDialect, NameGroupID) + ', ' +
      FormatIdentifier(FDataBase.SQLDialect, HashField) + ')');
    Add('values (' + IntToStr(ItemID) + ', ' + IntToStr(GrpId) +
      ', ' + IntToStr(ItemHash) + ')');
    try
      TransactionStart(FTransaction);
      Prepare;
      ExecQuery;
      TransactionCommit(FTransaction);
      RefreshItems;
      Result := True;
    except
      TransactionRollback(FTransaction);
      ErrorMsg(Format(sohoTreeItemAddError, [Name]));
    end;
  end;
  RestoreCursor;
end;

function TsohoIBTreeView.DeleteOneRecord(ItemID: Longint): boolean;
var aOnDel : TsohoTreeOnDeleteItem;
    Confirm: TModalResult;
begin
  case FDefAnswer of
    0: Result := true;
    1: begin
      Result := false;
      exit;
    end;
    else begin
      Result := false;
      aOnDel := OnItemDelete;
      if Assigned(aOnDel) then aOnDel(Self, ItemID, Result)
      else begin
        if FCanMany then begin
          Confirm := MessageDlg(
             sohoTreeDeleteCurrentItem,
             mtConfirmation, [mbYes, mbYesToAll, mbNo, mbNoToAll], 0);
          if Confirm = mrYesToAll then FDefAnswer := 0;
          if Confirm = mrNoToAll then FDefAnswer := 1;
          Result := Confirm in [mrYes, mrYesToAll];
        end
        else Result := YesNoMsg(sohoTreeDeleteCurrentItem);
      end;
    end;
  end;
  if not Result then exit;
  try
    with OperationQuery do begin
      Params[0].AsInteger := ItemID;
      try
        PushCursor(crHourGlass);
        ExecQuery;
        {         
                ! }
        Results.Remove(ItemID);
        Result := True;
      except ErrorMsg(Format(sohoTreeDeleteItemError, [Name]));
      end;
    end;
  finally
    RestoreCursor;
  end;
end;

function TsohoIBTreeView.DeleteItem: boolean;
var ID, index: Longint;
begin
  Result := False;
  if not (trDeleteItem in Options) then exit;
  if (ItemsDataSet = nil) or not (ItemsDataSet.Active) then exit;
  if ItemsDataSet.FieldByName(KeyField).IsNull then exit;
  ID := ItemsDataSet.FieldByName(KeyField).AsInteger;
  OperationQuery.DataBase := FDataBase;
  OperationQuery.Transaction := FTransaction;
  with OperationQuery, OperationQuery.SQL do begin
    Clear;
    Add('delete from ' + FormatIdentifier(FDataBase.SQLDialect, ItemsTable));
    Add('Where ' + FormatIdentifier(FDataBase.SQLDialect, KeyField) + '= ?Id');
  end;
  TransactionStart(FTransaction);
  FCanMany := Results.Count <> 0;
  FDefAnswer := 2;
  if Results.Count <> 0 then begin
    Result := False;
    {    ,      
           }
    Index := 0;
    while Index < Results.Count do begin
      if DeleteOneRecord(Results.Items[index]) then Result := True
      else Inc(index);
    end;
  end
  else Result := DeleteOneRecord(ID);
  {  -   ,    }
  if Result then begin
    TransactionCommit(FTransaction);
    RefreshItems;
    Changed;
  end;
end;

constructor TsohoIBTreeView.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  OperationQuery := TsohoIBSQL.Create(Self);
end;

function TsohoIBTreeView.RenumberRecords: boolean;
var index: Longint;
  Number: Longint;
  FMany : boolean;
begin
  {    }
  Result := True;
  FMany := ChildCheckBox.Checked;
  ChildCheckBox.Checked := False;
  Number := 0;
  if SaveFile <> '' then ;
    // SaveOutLineDescription(GroupOutLine, SaveFile);
  GroupTreeView.FullExpand;
  OperationQuery.DataBase := FDataBase;
  OperationQuery.Transaction := FTransaction;
  for Index := 0 to pred(GroupTreeView.Items.Count) do begin

    GroupTreeView.Selected := GroupTreeView.Items[Index];
    ItemsDataSet.First;
    try
      TransactionStart(FTransaction);
      while not ItemsDataSet.EOF do begin
        Inc(Number);
        with OperationQuery, SQL do begin
          Clear;
          Add('update ' + FormatIdentifier(FDataBase.SQLDialect, ItemsTable));
          Add('set ' +
             FormatIdentifier(FDataBase.SQLDialect, OrderField) + '=' + IntToStr(Number));
          Add('where ' +
             FormatIdentifier(FDataBase.SQLDialect, KeyField) + '=' +
                ItemsDataSet.FieldByName(KeyField).AsString);
          Prepare;
          ExecQuery;
        end;
        ItemsDataSet.Next;
      end;
      TransactionCommit(FTransaction);
      RefreshItems;
      try
        if SaveFile <> '' then ;
          // ApplyOutLineDescription(GroupOutLine, SaveFile);
      except
      end;
      ChildCheckBox.Checked := FMany;
    except
      TransactionRollback(FTransaction);
    end;
  end;
end;

destructor TsohoIBTreeView.Destroy;
begin
  OperationQuery.Free;
  inherited Destroy;
end;

procedure TsohoIBTreeView.MoveGroup (NewParentId, NewHash, Id : LongInt);
begin
  OperationQuery.DataBase := FDataBase;
  OperationQuery.Transaction := FTransaction;
  with OperationQuery do begin
   Close;
   SQL.Clear;
   SQL.Add('Update ' + FormatIdentifier(FDataBase.SQLDialect, GroupsTable));
   SQL.Add('Set ' +
     FormatIdentifier(FDataBase.SQLDialect, HashField) + '=' + IntToStr(NewHash) + ',');
   SQL.Add(FormatIdentifier(FDataBase.SQLDialect, NameParentID) + ' = ' + IntToStr(NewParentId));
   SQL.Add('Where ' +
     FormatIdentifier(FDataBase.SQLDialect, KeyField)+ '=' + IntToStr(Id));
   try
    PushCursor(crHourGlass);
    TransactionStart(FTransaction);
    Prepare;
    ExecQuery;
    TransactionCommit(FTransaction);
    RefreshItems;
   except
    TransactionRollback(FTransaction);
    ErrorMsg(Format(sohoTreeGroupMoveError,[Name]));
   end;
  end;
  RestoreCursor;
end;

function TsohoIBTreeView.GetNewItemHash;
var Temp: TsohoIBDataSet;
    TmpTr: TsohoIBTransaction;
begin
  Result := MaxLongint;
  if ItemsTable = '' then begin
    ErrorMsg(Format(sohoTreeHashGetError,[Name]));
    exit;
  end;
  Temp := TsohoIBDataSet.Create(Self);
  Temp.DataBase := FDataBase;
  TmpTr := TsohoIBTransaction.Create(nil);
  TmpTr.DefaultDatabase := FDataBase;
  Temp.Transaction := TmpTr;
  with Temp, Temp.SelectSQL do begin
    Clear;
    Add('select MAX(' +
      FormatIdentifier(FDataBase.SQLDialect, HashField) + ') from ' +
      FormatIdentifier(FDataBase.SQLDialect, ItemsTable));
    Add('where ' +
      FormatIdentifier(FDataBase.SQLDialect, NameGroupID) + '=' + IntToStr(GrpId));
    try
      TransactionStart(TmpTr);
      Prepare;
      Open;
      if EOF then Result := 0
      else Result := GetHash(Fields[0].AsInteger, MaxLongint);
      { Result = Fields[0].AsInteger  MaxLongInt, 
               }
      Close;
      Temp.Free;
    except ErrorMsg(Format(sohoTreeLastHashGetError,[Name]));
      Temp.Free;
    end;
    TransactionRollback(TmpTr);
    TmpTr.Free;
  end;
end;

function TsohoIBTreeView.CreateGroupsTree: boolean;
begin
  if (FDataBase = nil) and (SingleIBRegister<>nil) then
    FDataBase := SingleIBRegister.DataBase;
  if (FTransaction = nil) and (SingleIBRegister<>nil) then
    FTransaction := SingleIBRegister.Transaction;
  Result := inherited CreateGroupsTree;
end;

procedure TsohoIBTreeView.Notification(aComponent : TComponent; aOperation : TOperation);
begin
  if (aComponent = FTransaction) and (aOperation = opRemove) then
    FTransaction := nil;
  if (aComponent = FDataBase) and (aOperation = opRemove) then
    FDataBase := nil;
  inherited Notification(aComponent, aOperation);
end;

end.
