{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
{
         
  - InterBase
}
unit SoIBUtil;

{$I SOHOIBLIB.INC}

interface
uses Classes, Windows, SoIBClasses, FIB;

type

    TsohoIbConnectType = (ctLocal, ctTCPIP, ctNetBeui, ctIPX);

    {       }
    TsohoIBErrorPos = record
      Char, Line: LongInt;
    end;

    EIBError = EFIBError;

function TranslateIBError(const ErrorMessage: string): TsohoIBErrorPos;
{       }
function SetDataBase (DataBase : TsohoIBDatabase) : boolean;
{   ,       }
function ForgetDataBase : boolean;
{  }
function GetLocalIBPath : string;
{      Interbase }

function GetDataBaseNameInfo (var ServerName, FileName : string; var Protocol : TsohoIbConnectType) : boolean;
{          }
function ProtocolToStr (Protocol : TsohoIbConnectType) : string;
{    }

function TransactionStart (Tr : TsohoIBTransaction) : boolean;
{  ,      }
function NewTransactionStart (Tr : TsohoIBTransaction) : boolean;
{    }
function TransactionRollback(Tr : TsohoIBTransaction) : boolean;
{  ,    }
procedure TransactionCommit(Tr : TsohoIBTransaction);
{  ,    }

function GetLastSQLErrorCode : LongInt;
{     SQL }
function GetLastSQLErrorText : string;
{     SQL }
function GetLastSQLErrorPos  : TPoint;
{       SQL }

procedure SetLastError (E: EIBError);

var
   MetaDB      : TsohoIBDatabase;
   MetaTr      : TsohoIBTransaction;
   MetaQuery   : TsohoIBSQL;
   MetaDataSet : TsohoIBDataSet;

   LastSQLErrorCode : LongInt;
   LastSQlErrorText : ShortString;
   LastSQLErrorPos  : TPoint;

implementation
uses SysUtils, SoVCLUtils, DB, Registry, SoAppUtils, SoStrUtils;

ResourceString
   infoNoConnection = 'No active connection! Protocol and DataBase name are not defined!';

function GetDataBaseNameInfo (var ServerName, FileName : string; var Protocol : TsohoIbConnectType) : boolean;
var Tmp, Temp : string;
begin
  if (MetaDB = nil) or (not MetaDB.Connected) then begin
    Result := true;
    ErrorMsg(infoNoConnection);
    exit;
  end;
  Tmp := trim(MetaDB.DatabaseName);
  ServerName := 'LIB';
  FileName   := Tmp;
  if Tmp[2] = ':' then begin Protocol := ctLocal; end
  else begin
    // netBEUI Connection...
    if Pos('\\', Tmp) <> 0 then begin
      Temp     := Copy(Tmp, 3, Length(Tmp) - 2);
      ServerName   := Copy(Temp, 1, Pos('\', Temp) - 1);
      FileName := Copy(Temp, Pos('\', Temp) + 1, Length(Temp) - Pos('\', Temp));
      Protocol   := ctNetBeui;
    end;
    // TCP/IP Connection...
    if (Pos('@', Tmp) = 0) and (Pos('\\', Tmp) = 0) and
       (Pos(':', Tmp) <> 0) then begin
      ServerName   := Copy(Tmp, 1, Pos(':', Tmp) - 1);
      FileName := Copy(Tmp, Pos(':', Tmp) + 1, Length(Tmp) - Pos(':', Tmp));
      Protocol   := ctTCPIP;
    end;
    // IPX Connection...
    if Pos('@', Tmp) <> 0  then begin
      ServerName   := Copy(Tmp, 1, Pos('@', Tmp) - 1);
      FileName := Copy(Tmp, Pos('@', Tmp) + 1, Length(Tmp) - Pos('@', Tmp));
      Protocol := ctIPX;
    end;
  end;
  Result := true;
end;

function ProtocolToStr (Protocol : TsohoIbConnectType) : string;
begin
  case Protocol of    //
    ctLocal    : Result := 'Local';
    ctTCPIP    : Result := 'TCP/IP';
    ctNetBeui  : Result := 'NETBEUI';
    ctIPX      : Result := 'IPX';
  end;    // case
end;

function GetLocalIBPath : string;
const
  BORLAND_INTERBASE = '\SOFTWARE\Borland\InterBase\CurrentVersion';
  INTERBASE         = '\SOFTWARE\InterBase Corp\InterBase\CurrentVersion';
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Result := '';
    if Reg.OpenKey(BORLAND_INTERBASE, false) then
      Result := NormalDir(Reg.ReadString('RootDirectory'))
    else
      if Reg.OpenKey(INTERBASE, false) then
        Result := NormalDir(Reg.ReadString('RootDirectory'));
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

procedure SetLastError (E: EIBError);
var Trans: TsohoIBErrorPos;
begin
  LastSQLErrorCode := E.SQLCode;
  LastSQLErrorText := E.Message;
  Trans := TranslateIBError(E.Message);
  LastSQLErrorPos.X := Trans.Char;
  LastSQLErrorPos.Y := Trans.Line;
end;

function ForgetDataBase : boolean;
begin
  Result := MetaDB = nil;
  if Result then exit;
  MetaTr.Free;
  MetaQuery.Free;
  MetaDataSet.Free;
  MetaDB := nil;
  Result := true;
end;

function FindLineNumber (const Value : string) : integer;
var Len, x : integer;
    Tmp : string;
begin
  Result := -1;
  x := Pos('line', Value);
  Len := length(Value);
  while (x <= len) and (Value[x] <> ' ') do inc(x);
  if (x > len) then exit; // not found
  inc(x);
  Tmp := '';
  while (x <= len) and (Value[x] in ['0'..'9']) do begin
    Tmp := Tmp + Value[x];
    inc(x);
  end;
  Result := StrToIntDef(Tmp, 0);
end;

function FindCharNumber (const Value : string) : integer;
var Len, x : integer;
    Tmp : string;
begin
  Result := -1;
  x := Pos('char', Value);
  Len := length(Value);
  while (x <= len) and (Value[x] <> ' ') do inc(x);
  if (x > len) then exit; // not found
  inc(x);
  Tmp := '';
  while (x <= len) and (Value[x] in ['0'..'9']) do begin
    Tmp := Tmp + Value[x];
    inc(x);
  end;
  Result := StrToIntDef(Tmp, 0);
end;

function TranslateIBError(const ErrorMessage: string): TsohoIBErrorPos;
begin
  Result.Char := -1;
  Result.Line := -1;
  if pos('line', ErrorMessage) <> 0 then
    Result.Line := FindLineNumber(ErrorMessage);
  if pos('char', ErrorMessage) <> 0 then
    Result.Char := FindCharNumber(ErrorMessage);
end;

function SetDataBase (DataBase : TsohoIBDatabase) : boolean;
begin
  Result := (MetaDB = nil) or (DataBase = nil);
  if not Result then exit;
  MetaDB := DataBase;
  MetaTr := TsohoIBTransaction.Create(MetaDB.Owner);
  with MetaTr do begin
    DefaultDatabase := MetaDB;
    //     OnTimeOut  
  end;
  MetaQuery := TsohoIBSQL.Create(MetaDB.Owner);
  with MetaQuery do begin
    Database    := MetaDB;
    Transaction := MetaTr;
    ParamCheck := false;
  end;
  MetaDataSet := TsohoIBDataSet.Create(MetaDB.Owner);
  with MetaDataSet do begin
    Database    := MetaDB;
    Transaction := MetaTr;
  end;
end;

function NewTransactionStart (Tr : TsohoIBTransaction) : boolean;
begin
  try
    if Tr.InTransaction then Tr.Rollback;
    Tr.StartTransaction;
    Result := true;
  except
    Result := false;
  end;
end;

function TransactionStart (Tr : TsohoIBTransaction) : boolean;
begin
  try
    if not Tr.InTransaction then Tr.StartTransaction;
    Result := true;
  except
    Result := false;
  end;
end;

function TransactionRollback(Tr : TsohoIBTransaction) : boolean;
begin
  try
    if Tr.InTransaction then Tr.Rollback;
    Result := true;
  except
    Result := false;
  end;
end;

procedure TransactionCommit(Tr : TsohoIBTransaction);
begin
  if Tr.InTransaction then Tr.Commit;
end;

function GetLastSQLErrorCode : LongInt;
begin
  Result := LastSQLErrorCode;
end;

function GetLastSQLErrorText : string;
begin
  Result := LastSQLErrorText;
end;

function GetLastSQLErrorPos  : TPoint;
begin
  Result := LastSQLErrorPos;
end;

initialization
  MetaTr  := nil;
end.
