{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{               Written by Serg Vostrikov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
unit SoIbGrant;

{$I SOHOIBLIB.INC}
{$I FIBPLUS.INC}

interface
uses Classes, SoIBClasses, SoIBManager;


type

    PsohoIBUserInfo = ^TsohoIBUserInfo;
    {    }
    TsohoIBUserInfo = record
      UserName,
      GroupName,
      Comment,
      FirstName,
      MiddleName,
      LastName : TstrIBSystem;
    end;

    TsohoGrant = (grSelect, grUpdate, grDelete, grInsert, grExecute, grReference, grWithGrantOption);
    TsohoGrants = set of TsohoGrant;

    {  "",       .
          (), ,  
      .  ,       ,
           ,    
      utTable, , ,     }
    TsohoUserType = (utTable, utView, utTrigger, utProcedure, utUser, utRole);

    PsohoIbObjectGrantInfo = ^TsohoIbObjectGrantInfo;
    {        }
    TsohoIbObjectGrantInfo = record
      UserName : TstrIBSystem;
      UserType : TsohoUserType;
      ObjectType: TsohoUserType;
      Grantor  : TstrIBSystem;
      Grants : TsohoGrants;
    end;

    PsohoIBUserGrantInfo = ^TsohoIBUserGrantInfo;
    TsohoIBUserGrantInfo = record
      ObjectName : TstrIBSystem;
      UserType   : TsohoUserType;
      ObjectType: TsohoUserType;
      Grantor    : TstrIBSystem;
      Grants : TsohoGrants;
    end;

    TsohoIBUsersList = class(TList);
    TsohoIBGrantsList = class(TList);
    TsohoIBUserGrantsList = class(TList);

const
    GrantsNames: array [grSelect..grReference] of ShortString = ('S','U','D','I','X','R');
    FullGrantsName: array [grSelect..grReference] of ShortString =
      ('SELECT','UPDATE','DELETE','INSERT','EXECUTE', 'REFERENCES');
    ObjectTypeName: array [TsohoUserType] of ShortString =
      ('table', 'view', 'trigger', 'procedure', '', '');
    ObjectTypeIdent: array [TsohoUserType] of Integer = (0, 1, 2, 5, 8, 13);


type

  TsohoIBGrantManager = class(TComponent)
  private
    FDatabase: TsohoIBDatabase;
    FTransaction: TsohoIBTransaction;
    FQuery: TsohoIBSQL;

    FIsc4Path: string;
    FIsc4DB: TsohoIBDatabase;
    FIscTr: TsohoIBTransaction;
    FIscQuery: TsohoIBSQL;

    FItems: TStringList;

    FAutoCommit: boolean;
    function GetItems: TStrings;
    procedure SetDatabase(const Value: TsohoIBDatabase);
    procedure SetIsc4Path(const Value: string);
  protected
    function TryToGetUsersBase : string;
    function FillObjectItems: boolean;
    function ExecIscQuery (const QueryText: string; ShowError: boolean): boolean;
    function ExecGrantQuery (const QueryText: string; ShowError: boolean): boolean;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    function Isc4Connected: boolean;
    procedure ConnectIsc4;

    {     }
    function GetUsersNamesList: boolean;
    {    }
    function GetRolesList: boolean;
    {   ,     }
    function GetUserRolesList(const UserName: string): boolean;
    {     }
    function GetGroupsList: boolean;

    {     }
    function GetUserInfo (const UserName : TstrIBSystem) : PsohoIBUserInfo;
    {     }
    function SetUserInfo (UserInfo : TsohoIBUserInfo; const OldName, Password : TstrIBSystem) : boolean;
    {   }
    function AddUser (UserInfo : TsohoIBUserInfo; const Password : TstrIBSystem) : boolean;
    {   }
    function DeleteUser (const UserName : TstrIBSystem) : boolean;
    {   }
    function AddRole(const RoleName: string): boolean;
    {   }
    function DeleteRole (const RoleName: string): boolean;

    {     }
    function AddUserToRole(const UserName, RoleName: string; WithAdminOption: boolean): boolean;
    {     }
    function RemoveUserFromRole(const UserName, RoleName: string): boolean;

    {    }
    function  CreateUsersList : TsohoIBUsersList;
    {    }
    procedure FreeUsersList (List : TsohoIBUsersList);

    {       }
    function GrantsSetToStr(Grants : TsohoGrants) : TstrResult;
    {       }
    function GrantsNameToGrants (const GrantName : TstrResult) : TsohoGrant;
    {        }
    function CreateGrantsList (const GrantObjectName : TstrIBSystem): TsohoIBGrantsList;
    {        }
    procedure FreeGrantsList (var List : TsohoIBGrantsList);
    {    api   5. }
    function IsSecurityAPIAviable : boolean;
    {     /   }
    function  CreateUserGrantsList (const UserName : TstrIBSystem; UserType: TsohoUserType): TsohoIBUserGrantsList;
    {      /   }
    procedure FreeUserGrantsList (var List: TsohoIBUserGrantsList);

    {         }
    function SetGrantsCategoryForObject (const GrantObjectName,
      UserName : TstrIBSystem; UserType: TsohoUserType; Grants: TsohoGrant; SetOn: boolean): boolean;
    {           }
    function SetAllGrantsForObject (const GrantObjectName,
      UserName : TstrIBSystem; UserType: TsohoUserType; IsObjectProcedure, SetOn: boolean): boolean;

    property GrantsTransaction: TsohoIBTransaction read FTransaction;      
  published
    property Items: TStrings read GetItems;
    property Database: TsohoIBDatabase read FDatabase write SetDatabase;
    property Isc4Path: string read FIsc4Path write SetIsc4Path;
    property AutoCommit: boolean read FAutoCommit write FAutoCommit default false;
  end;
                           
implementation
uses SysUtils, SoVCLUtils, SoAppUtils, Windows, SoRgstry, Unxcrypt,
     IBASE, ib_externals, SoStrUtils, Registry;

type

    TUserProc = function (status_vector             : PISC_STATUS;
                          user_sec_data             : PUserSecData): ISC_STATUS;

var
    AddProc,
    ModifyProc,
    DeleteProc : TUserProc;

function GetLocalIBPath : string;
const
  BORLAND_INTERBASE = '\SOFTWARE\Borland\InterBase\CurrentVersion';
  INTERBASE         = '\SOFTWARE\InterBase Corp\InterBase\CurrentVersion';
var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Result := '';
    if Reg.OpenKey(BORLAND_INTERBASE, false) then
      Result := NormalDir(Reg.ReadString('RootDirectory'))
    else
      if Reg.OpenKey(INTERBASE, false) then
        Result := NormalDir(Reg.ReadString('RootDirectory'));
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;

function GetUserType(UserType: Integer): TsohoUserType;
begin
  case UserType of
    5: Result := utProcedure;
    2: Result := utTrigger;
    1: Result := utView;
    else Result := utUser;
  end;
end;

procedure FindUserFunctions;
var ISC_Handle : hModule;
begin
  AddProc := nil;
  ModifyProc := nil;
  DeleteProc := nil;
  ISC_Handle := GetModuleHandle(IBASE_DLL);
  if ISC_Handle = 0 then exit;
  AddProc    := GetProcAddress(ISC_Handle, 'isc_add_user');
  ModifyProc := GetProcAddress(ISC_Handle, 'isc_modify_user');
  DeleteProc := GetProcAddress(ISC_Handle, 'isc_delete_user');
end;

{ TsohoIBGrantManager }
function TsohoIBGrantManager.AddRole(const RoleName: string): boolean;
begin
  Result := ExecGrantQuery('create role ' + RoleName, false);
end;

function TsohoIBGrantManager.AddUser(UserInfo: TsohoIBUserInfo;
  const Password: TstrIBSystem): boolean;
var Strm : TMemoryStream;
    Buffer : pointer;
begin
  Result := FIsc4DB <> nil;
  if not Result then exit;

  PushCursor(crHourGlass);

  if not FIscTr.Active then FIscTr.StartTransaction;

  with FIscQuery, SQL do begin
    try
      Clear;
      ParamCheck := true;
      Add('insert into USERS (USER_NAME, GROUP_NAME, FIRST_NAME, MIDDLE_NAME, LAST_NAME,');
      Add('COMMENT, PASSWD)');
      Add('values (?USER_NAME, ?GROUP_NAME, ?FIRST_NAME, ?MIDDLE_NAME, ?LAST_NAME,');
      Add('?COMMENT, ?PASSWD)');
      Strm := TMemoryStream.Create;
      if length(UserInfo.Comment) > 0 then begin
        GetMem(Buffer, length(UserInfo.Comment)+1);
        StrPCopy(Buffer, UserInfo.Comment);
        Strm.Write(Buffer, length(UserInfo.Comment));
        FreeMem(Buffer, length(UserInfo.Comment)+1);
      end;
      Strm.Position := 0;
      with UserInfo do begin
        Params[0].AsString := UserName;
        Params[1].AsString := GroupName;
        Params[2].AsString := FirstName;
        Params[3].AsString := MiddleName;
        Params[4].AsString := LastName;
        Params[5].LoadFromStream(Strm);
        Params[6].AsString := CreateInterbasePassword(Password);
      end;
      ExecQuery;
      Strm.Free;
      FIscTr.Commit;
      RestoreCursor;
    except
      on E: EIBError do begin
        RestoreCursor;
        FIscTr.Rollback;
        ErrorMsg(E.Message);
        Result := false;
      end;
    end;
    ParamCheck := false;
  end;
end;

function TsohoIBGrantManager.AddUserToRole(const UserName,
  RoleName: string; WithAdminOption: boolean): boolean;
const GrantSQL = 'grant %s  to %s%s';
begin
  if WithAdminOption then
    Result := ExecGrantQuery(Format(GrantSQL, [RoleName, UserName, ' with admin option']), false)
  else
    Result := ExecGrantQuery(Format(GrantSQL, [RoleName, UserName, ' ']), false);
end;

constructor TsohoIBGrantManager.Create(aOwner: TComponent);
begin
  inherited;
  FIsc4Path := '';
  FIsc4DB := TsohoIBDatabase.Create(Self);
  FIscTr := TsohoIBTransaction.Create(Self);
  FIscTr.DefaultDatabase := FIsc4DB;
  FIscQuery := TsohoIBSQL.Create(Self);
  with FIscQuery do begin
    Transaction := FIscTr;
    Database := FIsc4DB;
  end;

  FTransaction := TsohoIBTransaction.Create(Self);
  FQuery := TsohoIBSQL.Create(Self);
  FQuery.Transaction := FTransaction;

  FItems := TStringList.Create;
end;

function TsohoIBGrantManager.CreateGrantsList(
  const GrantObjectName: TstrIBSystem): TsohoIBGrantsList;
var UserInfo : PsohoIbObjectGrantInfo;
    NewName, LastName : string;
begin
  Result := nil;
  if FDatabase = nil then exit;

  UserInfo := nil;
  PushCursor(crHourGlass);

  if not FTransaction.Active then FTransaction.StartTransaction;
  with FQuery, SQL do begin
    try
      Clear;
      Add('select * from RDB$USER_PRIVILEGES where RDB$RELATION_NAME = ' +
         SingleQuotes(GrantObjectName) + ' order by RDB$USER, RDB$OBJECT_TYPE');
      ExecSQL;
      LastName := '';
      if not Eof then Result := TsohoIBGrantsList.Create;
      while not Eof do begin
        {$IFNDEF FIELD_BY_NAME_AS_FUNC}
        NewName := trim(FieldByName['RDB$USER'].AsString);
        {$ELSE}
        NewName := trim(FieldByName('RDB$USER').AsString);
        {$ENDIF}
        if LastName <> NewName then begin
           New(UserInfo);
           LastName := NewName;
           with UserInfo^ do begin
             UserName := NewName;
             {$IFNDEF FIELD_BY_NAME_AS_FUNC}
             Grantor := trim(FieldByName['RDB$GRANTOR'].AsString);
             Grants := [GrantsNameToGrants(trim(FieldByName['RDB$PRIVILEGE'].AsString))];
             UserType := GetUserType(FieldByName['RDB$USER_TYPE'].AsInteger);
             ObjectType := GetUserType(FieldByName['RDB$OBJECT_TYPE'].AsInteger);
             {$ELSE}
             Grantor := trim(FieldByName('RDB$GRANTOR').AsString);
             Grants := [GrantsNameToGrants(trim(FieldByName('RDB$PRIVILEGE').AsString))];
             UserType := GetUserType(FieldByName('RDB$USER_TYPE').AsInteger);
             ObjectType := GetUserType(FieldByName('RDB$OBJECT_TYPE').AsInteger);
             if FieldByName('RDB$GRANT_OPTION').AsInteger = 1 then
               Grants := Grants + [grWithGrantOption];
             {$ENDIF}
           end;    // with
           Result.Add(UserInfo);
        end
        else begin
           UserInfo^.Grants := UserInfo^.Grants +
           {$IFNDEF FIELD_BY_NAME_AS_FUNC}
             [GrantsNameToGrants(trim(FieldByName['RDB$PRIVILEGE'].AsString))];
           if FieldByName['RDB$GRANT_OPTION'].AsInteger = 1 then
             UserInfo^.Grants := UserInfo^.Grants + [grWithGrantOption];
           {$ELSE}
             [GrantsNameToGrants(trim(FieldByName('RDB$PRIVILEGE').AsString))];
           if FieldByName('RDB$GRANT_OPTION').AsInteger = 1 then
             UserInfo^.Grants := UserInfo^.Grants + [grWithGrantOption];
           {$ENDIF}
        end;
        Next;
      end;
      Close;
      if FAutoCommit then FTransaction.Commit;
      RestoreCursor;
    except
      on E: EIBError do begin
        if FAutoCommit then FTransaction.Rollback;
        RestoreCursor;
        ErrorMsg(E.Message);
        Result := nil;
      end;
    end;
  end;
end;

function TsohoIBGrantManager.CreateUserGrantsList(
  const UserName: TstrIBSystem;
  UserType: TsohoUserType): TsohoIBUserGrantsList;
var ObjectInfo : PsohoIBUserGrantInfo;
    NewName, LastName : string;
begin
  Result := nil;
  if FDatabase = nil then exit;

  ObjectInfo := nil;
  PushCursor(crHourGlass);

  if not FTransaction.Active then FTransaction.StartTransaction;
  
  with FQuery, SQL do begin
    try
      Clear;
      Add('select * from RDB$USER_PRIVILEGES where (RDB$USER = '+
          SingleQuotes(UserName) + ') and (RDB$USER_TYPE = ' +
          IntToStr(ObjectTypeIdent[UserType])+
          ') order by RDB$OBJECT_TYPE, RDB$RELATION_NAME');
      ExecSQL;
      LastName := '';
      if not Eof then
        Result := TsohoIBUserGrantsList.Create;
      while not Eof do begin
        {$IFNDEF FIELD_BY_NAME_AS_FUNC}
        NewName := trim(FieldByName['RDB$RELATION_NAME'].AsString);
        {$ELSE}
        NewName := trim(FieldByName('RDB$RELATION_NAME').AsString);
        {$ENDIF}
        if LastName <> NewName then begin
           New(ObjectInfo);
           LastName := NewName;
           with ObjectInfo^ do begin
             ObjectName := NewName;
             {$IFNDEF FIELD_BY_NAME_AS_FUNC}
             Grantor := trim(FieldByName['RDB$GRANTOR'].AsString);
             Grants := [GrantsNameToGrants(trim(FieldByName['RDB$PRIVILEGE'].AsString))];
             UserType := GetUserType(FieldByName['RDB$USER_TYPE'].AsInteger);
             ObjectType := GetUserType(FieldByName['RDB$OBJECT_TYPE'].AsInteger);
             if FieldByName['RDB$GRANT_OPTION'].AsInteger = 1 then
               Grants := Grants + [grWithGrantOption];
             {$ELSE}
             Grantor := trim(FieldByName('RDB$GRANTOR').AsString);
             Grants := [GrantsNameToGrants(trim(FieldByName('RDB$PRIVILEGE').AsString))];
             UserType := GetUserType(FieldByName('RDB$USER_TYPE').AsInteger);
             ObjectType := GetUserType(FieldByName('RDB$OBJECT_TYPE').AsInteger);
             if FieldByName('RDB$GRANT_OPTION').AsInteger = 1 then
               Grants := Grants + [grWithGrantOption];
             {$ENDIF}
           end;    // with
           Result.Add(ObjectInfo);
        end
        else begin
           {$IFNDEF FIELD_BY_NAME_AS_FUNC}
           ObjectInfo^.Grants := ObjectInfo^.Grants + [GrantsNameToGrants(trim(FieldByName['RDB$PRIVILEGE'].AsString))];
           if FieldByName['RDB$GRANT_OPTION'].AsInteger = 1 then
             ObjectInfo^.Grants := ObjectInfo^.Grants + [grWithGrantOption];
           {$ELSE}
           ObjectInfo^.Grants := ObjectInfo^.Grants + [GrantsNameToGrants(trim(FieldByName('RDB$PRIVILEGE').AsString))];
           if FieldByName('RDB$GRANT_OPTION').AsInteger = 1 then
             ObjectInfo^.Grants := ObjectInfo^.Grants + [grWithGrantOption];
           {$ENDIF}
        end;
        Next;
      end;
      Close;
      if FAutoCommit then FTransaction.Commit;
      RestoreCursor;
    except
      on E: EIBError do begin
        if FAutoCommit then FTransaction.Rollback;
        RestoreCursor;
        ErrorMsg(E.Message);
        Result := nil;
      end;
    end;
  end;
end;

function TsohoIBGrantManager.CreateUsersList: TsohoIBUsersList;
var UserInfo : PsohoIbUserInfo;
begin
  Result := nil;
  if FIsc4DB = nil then exit;

  with FIscQuery, SQL do begin
    if not FIscTr.Active then FIscTr.StartTransaction;
    try
      Clear;
      Add('select * from USERS order by USER_NAME');
      ExecSQL;
      Result := TsohoIBUsersList.Create;
      while not Eof do begin
        New(UserInfo);
        with UserInfo^ do begin
          {$IFNDEF FIELD_BY_NAME_AS_FUNC}
          UserName   := FieldByName['USER_NAME'].AsString;
          GroupName  := FieldByName['GROUP_NAME'].AsString;
          Comment    := FieldByName['COMMENT'].AsString;
          FirstName  := FieldByName['FIRST_NAME'].AsString;
          MiddleName := FieldByName['MIDDLE_NAME'].AsString;
          LastName   := FieldByName['LAST_NAME'].AsString;
          {$ELSE}
          UserName   := FieldByName('USER_NAME').AsString;
          GroupName  := FieldByName('GROUP_NAME').AsString;
          Comment    := FieldByName('COMMENT').AsString;
          FirstName  := FieldByName('FIRST_NAME').AsString;
          MiddleName := FieldByName('MIDDLE_NAME').AsString;
          LastName   := FieldByName('LAST_NAME').AsString;
          {$ENDIF}
        end;
        Result.Add(UserInfo);
        Next;
      end;
      FIscTr.Commit;
    except
      FIscTr.Rollback;
      Result := nil;
    end;
  end;
end;

function TsohoIBGrantManager.DeleteRole(const RoleName: string): boolean;
begin
  Result := ExecGrantQuery('drop role ' + RoleName, false);
end;

function TsohoIBGrantManager.DeleteUser(
  const UserName: TstrIBSystem): boolean;
begin
  Result := FIsc4DB <> nil;
  if not Result then exit;

  Result := ExecIscQuery('delete from Users where USER_NAME = ' +
    SingleQuotes(UserName), true);
end;

destructor TsohoIBGrantManager.Destroy;
begin
  FTransaction.Rollback;
  FQuery.Free;
  FTransaction.Free;

  FIscTr.Commit;
  FIscQuery.Free;
  FIsc4DB.Close;
  FIsc4DB.Free;

  FItems.Free;
  inherited;
end;

function TsohoIBGrantManager.FillObjectItems: boolean;
begin
  Result := FDatabase <> nil;
  if not Result then exit;

  //       ,    isc4
  with FQuery, SQL do begin
    if not FTransaction.Active then FTransaction.StartTransaction;

    try
      FItems.Clear;
      ExecSQL;
      while not Eof do begin
        FItems.Add(trim(Fields[0].AsString));
        Next;
      end;
      if FAutoCommit then FTransaction.Commit;
    except
      on E: EIBError do begin
        Result := false;
        if FAutoCommit then FTransaction.Rollback;
      end;
    end;
    Close;
  end;
end;

procedure TsohoIBGrantManager.FreeGrantsList(var List: TsohoIBGrantsList);
var Index : integer;
begin
  for Index := 0 to pred(List.Count) do
    Dispose(PsohoIbObjectGrantInfo(List[Index]));
  List.Free;
  List := nil;
end;

procedure TsohoIBGrantManager.FreeUserGrantsList(
  var List: TsohoIBUserGrantsList);
var Index : integer;
begin
  for Index := 0 to pred(List.Count) do
    Dispose(PsohoIBUserGrantInfo(List[Index]));
  List.Free;
  List := nil;
end;

procedure TsohoIBGrantManager.FreeUsersList(List: TsohoIBUsersList);
var Index : integer;
begin
  if List = nil then exit;
  for Index := 0 to pred(List.Count) do Dispose(PsohoIBUserInfo(List[Index]));
  List.Free;
end;

function TsohoIBGrantManager.GetGroupsList: boolean;
begin
  Result := FIsc4DB <> nil;
  if not Result then exit;

  with FIscQuery, SQL do begin
    if not FIscTr.Active then FIscTr.StartTransaction;
    try
      Clear;
      FItems.Clear;
      Add('select GROUP_NAME from USERS group by GROUP_NAME');
      ExecSQL;
      while not Eof do begin
        FItems.Add(trim(Fields[0].AsString));
        Next;
      end;
      FIscTr.Commit;
    except
      on E: EIBError do begin
        FIscTr.Rollback;
        Result := false;
        ErrorMsg(E.Message);
      end;
    end;
  end;
end;

function TsohoIBGrantManager.GetItems: TStrings;
begin
  Result := TStrings(FItems);
end;

function TsohoIBGrantManager.GetRolesList: boolean;
begin
  Result := FDatabase <> nil;
  if not Result then exit;

  //       ,    isc4
  with FQuery, SQL do begin
    Clear;
    Add('select RDB$ROLE_NAME from RDB$ROLES order by RDB$ROLE_NAME');
  end;

  Result := FillObjectItems;
end;

function TsohoIBGrantManager.GetUserInfo(
  const UserName: TstrIBSystem): PsohoIBUserInfo;
begin
  Result := nil;
  if FIsc4DB = nil then exit;

  with FIscQuery, SQL do begin
    if not FIscTr.Active then exit;
    try
      Clear;
      Add('select * from USERS where USER_NAME = '+
        SingleQuotes(UserName));
      ExecSQL;
      New(Result);
      with Result^ do begin
        {$IFNDEF FIELD_BY_NAME_AS_FUNC}
        UserName   := FieldByName['USER_NAME'].AsString;
        GroupName  := FieldByName['GROUP_NAME'].AsString;
        Comment    := FieldByName['COMMENT'].AsString;
        FirstName  := FieldByName['FIRST_NAME'].AsString;
        MiddleName := FieldByName['MIDDLE_NAME'].AsString;
        LastName   := FieldByName['LAST_NAME'].AsString;
        {$ELSE}
        UserName   := FieldByName('USER_NAME').AsString;
        GroupName  := FieldByName('GROUP_NAME').AsString;
        Comment    := FieldByName('COMMENT').AsString;
        FirstName  := FieldByName('FIRST_NAME').AsString;
        MiddleName := FieldByName('MIDDLE_NAME').AsString;
        LastName   := FieldByName('LAST_NAME').AsString;
        {$ENDIF}
      end;
      FIscTr.Commit;
    except
      FIscTr.Rollback;
      Result := nil;
    end;
  end;
end;

function TsohoIBGrantManager.GetUserRolesList(
  const UserName: string): boolean;
begin
  Result := FDatabase <> nil;
  if not Result then exit;

  //       ,    isc4
  with FQuery, SQL do begin
    Clear;
    Add('select RDB$RELATION_NAME from RDB$USER_PRIVILEGES');
    Add('where RDB$USER = ' + SingleQuotes(AnsiUpperCase(UserName)) +
        ' and RDB$PRIVILEGE = ' + SingleQuotes('M'));
    Add('order by RDB$RELATION_NAME');
  end;

  Result := FillObjectItems;
end;

function TsohoIBGrantManager.GetUsersNamesList: boolean;
begin
  Result := FIsc4DB <> nil;
  if not Result then exit;

  with FIscQuery, SQL do begin
    if not FIscTr.InTransaction then FIscTr.StartTransaction;
    try
      Clear;
      FItems.Clear;
      Add('select USER_NAME from USERS order by USER_NAME');
      ExecSQL;
      while not Eof do begin
        FItems.Add(trim(Fields[0].AsString));
        Next;
      end;
      FItems.Add('PUBLIC');
      FIscTr.Commit;
    except
      on E: EIBError do begin
        Result := false;
        FIscTr.Rollback;
        ErrorMsg(E.Message);
      end;
    end;
  end;
end;

function TsohoIBGrantManager.GrantsNameToGrants(
  const GrantName: TstrResult): TsohoGrant;
var Index : TsohoGrant;
begin
  Result := grSelect;
  for Index := grSelect to grReference do
   if GrantsNames[Index] = GrantName then begin
     Result := Index;
     exit;
   end;
end;

function TsohoIBGrantManager.GrantsSetToStr(
  Grants: TsohoGrants): TstrResult;
var Index : TsohoGrant;
begin
  Result := EmptyStr;
  for Index := grSelect to grReference do
    if Index in Grants then Result := Result + ', '+GrantsNames[Index];
  if Result <> EmptyStr then Result := copy(Result, 3, length(Result));
end;

function TsohoIBGrantManager.Isc4Connected: boolean;
begin
  Result := FIsc4DB.Connected;
end;

function TsohoIBGrantManager.IsSecurityAPIAviable: boolean;
begin
  Result := (@AddProc <> nil);
end;

function TsohoIBGrantManager.RemoveUserFromRole(const UserName,
  RoleName: string): boolean;
begin
  Result := ExecGrantQuery('revoke ' + RoleName + ' from ' + UserName, false);
end;

function TsohoIBGrantManager.SetAllGrantsForObject(const GrantObjectName,
  UserName: TstrIBSystem; UserType: TsohoUserType; IsObjectProcedure,
  SetOn: boolean): boolean;
begin
  Result := FDatabase <> nil;
  if not Result then exit;

  PushCursor(crHourGlass);

  if not FTransaction.Active then FTransaction.StartTransaction;

  with FQuery, SQL do begin
    try
      Clear;
      if SetOn then begin
        if IsObjectProcedure then
          Add('grant execute on procedure '+GrantObjectName)
        else
          Add('grant all on '+GrantObjectName);
        Add('to '+ObjectTypeName[UserType]+' '+UserName{+' WITH GRANT OPTION'});
      end
      else begin
        if IsObjectProcedure then
          Add('revoke execute on procedure '+GrantObjectName)
        else
          Add('revoke all on '+GrantObjectName);
        Add('from '+ObjectTypeName[UserType]+' '+UserName);
      end;
      ExecSQL;
      if FAutoCommit then FTransaction.Commit;
      RestoreCursor;
    except
      on E: EIBError do begin
        Result := false;
        if FAutoCommit then FTransaction.Rollback;
        RestoreCursor;
        ErrorMsg(E.Message);
      end;
    end;
  end;
end;

procedure TsohoIBGrantManager.SetDatabase(const Value: TsohoIBDatabase);
begin
  if FDatabase <> nil then begin
    FTransaction.Rollback;
    if Isc4Connected then FIsc4DB.Close;
  end;
  FDatabase := Value;
  FTransaction.DefaultDatabase := FDatabase;
  FQuery.Database := FDatabase;
end;

function TsohoIBGrantManager.SetGrantsCategoryForObject(
  const GrantObjectName, UserName: TstrIBSystem; UserType: TsohoUserType;
  Grants: TsohoGrant; SetOn: boolean): boolean;
begin
  Result := FDatabase <> nil;
  if not Result then exit;

  PushCursor(crHourGlass);

  if not FTransaction.Active then FTransaction.StartTransaction;
  with FQuery, SQL do begin
    try
      Clear;
      if SetOn then begin
        if Grants = grExecute then
          Add('grant '+FullGrantsName[Grants]+' on procedure '+GrantObjectName)
        else
          Add('grant '+FullGrantsName[Grants]+' on '+GrantObjectName);
        Add('to '+ObjectTypeName[UserType]+' '+UserName);
      end
      else begin
        if Grants = grExecute then
          Add('revoke '+FullGrantsName[Grants]+' on procedure '+GrantObjectName)
        else
          Add('revoke '+FullGrantsName[Grants]+' on '+GrantObjectName);
        Add('from '+ObjectTypeName[UserType]+' '+UserName);
      end;
      ExecSQL;
      if FAutoCommit then FTransaction.Commit;
      RestoreCursor;
    except
      on E: EIBError do begin
        Result := false;
        if FAutoCommit then FTransaction.Rollback;
        RestoreCursor;
        ErrorMsg(E.Message);
      end;
    end;
  end;
end;

procedure TsohoIBGrantManager.SetIsc4Path(const Value: string);
begin
  FIsc4Path := Value;
  FIsc4DB.Close;
end;

function TsohoIBGrantManager.SetUserInfo(UserInfo: TsohoIBUserInfo;
  const OldName, Password: TstrIBSystem): boolean;
var Strm : TMemoryStream;
    Desc : TstringList;
begin
  Result := (FIsc4DB <> nil) and (Password <> EmptyStr);
  if not Result then exit;

  PushCursor(crHourGlass);
  if not FIscTr.Active then FIscTr.StartTransaction;

  with FIscQuery, SQL do begin
    try
      Clear;
      ParamCheck := true;
      Add('update USERS set ');
      Add(' GROUP_NAME = ?GN,');
      Add(' FIRST_NAME = ?FN,');
      Add(' MIDDLE_NAME = ?MN,');
      Add(' LAST_NAME  = ?LN,');
      Add(' PASSWD = ?PW,');
      Add(' COMMENT = ?CM');
      Add('where USER_NAME = ' + SingleQuotes(OldName));
      Desc := TstringList.Create;
      Desc.Text := UserInfo.Comment;
      Strm := TMemoryStream.Create;
      Desc.SaveToStream(Strm);
      Strm.Position := 0;
      Desc.Free;
      with UserInfo do begin
        Params[0].AsString := GroupName;
        Params[1].AsString := FirstName;
        Params[2].AsString := MiddleName;
        Params[3].AsString := LastName;
        Params[4].AsString := CreateInterbasePassword(Password);
        Params[5].LoadFromStream(Strm);
      end;
      ExecQuery;
      Strm.Free;
      FIscTr.Commit;
      RestoreCursor;
    except
      on E: EIBError do begin
        RestoreCursor;
        ErrorMsg(E.Message);
        FIscTr.Rollback;
        Result := false;
      end;
    end;
    ParamCheck := false;
  end;
end;

function TsohoIBGrantManager.TryToGetUsersBase: string;
begin
  Result := GetLocalIBPath;
  if Result <> EmptyStr then begin
    Result := Result+'isc4.gdb';
    if not FileExists(Result) then Result := EmptyStr;
  end;
end;

function TsohoIBGrantManager.ExecIscQuery(const QueryText: string; ShowError: boolean): boolean;
begin
  Result := FIsc4DB <> nil;
  if not Result then exit;

  PushCursor(crHourGlass);
  if not FIscTr.Active then FIscTr.StartTransaction;
  try
    with FIscQuery, SQL do begin
      Clear;
      Text := QueryText;
      ExecSQL;
    end;
    FIscTr.Commit;
    RestoreCursor;
  except
    on E: EIBError do begin
      FIscTr.Rollback;
      RestoreCursor;
      Result := false;
      if ShowError then ErrorMsg(E.Message);
    end;
  end;
end;

function TsohoIBGrantManager.ExecGrantQuery(const QueryText: string;
  ShowError: boolean): boolean;
begin
  Result := FDatabase <> nil;
  if not Result then exit;

  if not FTransaction.Active then FTransaction.StartTransaction;
  with FQuery do begin
    try
      SQL.Text := QueryText;
      ExecSQL;
      FTransaction.Commit;
    except
      on E: EIBError do begin
        RestoreCursor;
        FTransaction.Rollback;
        Result := false;
        if ShowError then ErrorMsg(E.Message);
      end;
    end;
  end;
end;

procedure TsohoIBGrantManager.ConnectIsc4;
begin
  if FDatabase = nil then exit;

  try
    FIsc4DB.DatabaseName := FIsc4Path;
    FIsc4DB.UserName := FDatabase.UserName;
    FIsc4DB.Password := FDatabase.Password;
    FIsc4DB.Open;
  except
  end;
end;

initialization
  FindUserFunctions;
finalization
end.
