{*******************************************************}
{                                                       }
{           Delphi Visual Component Library             }
{                                                       }
{           Copyright (c) 2000-2001 Devrace             }
{              Written by Igor Chernyshov               }
{                                                       }
{            Devrace SohoIBLib Pro Components           }
{           This library is a trustware product         }
{            Please, read the License.txt file          }
{                                                       }
{             url   : http://www.devrace.com/           }
{             e-mail: sohoiblib@devrace.com             }
{                                                       }
{*******************************************************}
{
       .   
    :)

   TsohoIBAssist?
  ,    
   /  .   -
 ,      Edit`,    ,  
 , - ,  Insert/Update.    
   -  ,      sohoIBAssist.

   sohoIBAssist?
   ,   DataSet .   ,
   /  FieldsList.     
   EditsList   ,   .
      TableName    KeyField.
       Edit  ReadData, 
 /  - InsertData  UpdateData
 TsohoIBAssist     :
 TsohoLabel      -   Caption
 TsohoEdit       -  Text
 TsohoSpinEdit   -  Value
 TsohoCheckBox   -  State
 TsohoComboBox   -  Text
 TsohoIBComboBox -  ItemID
 TCustomMemo     -  Lines.Text
 TCustomRichEdit -  Lines.Text
}
unit SohoIBAssist;

{$I SOHOIBLIB.INC}

interface
uses SysUtils, Classes, DB, SoIBClasses, SoCtmDBAssist, SoCtrls,
     Controls;

type

  TsohoIBAssist = class(TsohoCustomDBAssist)
  private
    FQuery: TsohoIBSQL;
    FGeneratorName: string;
  protected
    procedure Notification (aComponent: TComponent; aOperation: TOperation); override;
    procedure SetDataSet(Value : TDataSet); override;
    procedure ExecuteInsertSQL; override;
    procedure ExecuteUpdateSQL; override;
    procedure SetStrParam(const ParamName: string; const Value: string); override;
    procedure SetIntParam(const ParamName: string; const Value: LongInt); override;
    procedure SetFltParam(const ParamName: string; const Value: Double); override;
    function TypeRequest(Edit: TsohoCustomEdit): integer; override;
    procedure SetDateParam(const ParamName: string; const Value: TDate); override;
    procedure SetTimeParam(const ParamName: string; const Value: TTime); override;
    procedure SetDateTimeParam(const ParamName: string; const Value: TDateTime); override;
    procedure SetParamNull(const ParamName: string); override;
    function ParamExists(const ParamName: string): boolean; override;
    function GetTableName: string; override;
    function GetFieldName(const FieldName: string): string; override;
    procedure PrepareQuery; override;
  public
    {  -      }
    constructor Create(AOwner: TComponent); override;
    {   Destroy }
    destructor Destroy; override;
    procedure InsertData; override;
  published
    {   -  ,  
            KeyField.
         ,      
      KeyValue    InsertKeyField }
    property GeneratorName: string read FGeneratorName write FGeneratorName;
  end;

implementation
uses SoStrUtils, SoUnit;

{ TsohoIBAssist }

constructor TsohoIBAssist.Create(AOwner: TComponent);
begin
  inherited;
  FGeneratorName := EmptyStr;
  FQuery := TsohoIBSQL.Create(Self);
end;

destructor TsohoIBAssist.Destroy;
begin
  FQuery.Free;
  inherited;
end;

procedure TsohoIBAssist.ExecuteInsertSQL;
begin
  try
    FQuery.Prepare;
    FQuery.ExecSQL;
    FQuery.Close;
  except
  end;
end;

procedure TsohoIBAssist.ExecuteUpdateSQL;
begin
  ExecuteInsertSQL;
end;

function TsohoIBAssist.GetFieldName(const FieldName: string): string;
begin
  Result := FormatIdentifier(FQuery.Database.SQLDialect, FieldName);
end;

function TsohoIBAssist.GetTableName: string;
begin
  Result := FormatIdentifier(FQuery.Database.SQLDialect, TableName);
end;

procedure TsohoIBAssist.Notification(aComponent: TComponent;
  aOperation: TOperation);
begin
  if (aComponent = DataSet) and (aOperation = opRemove) then begin
    FQuery.Database := nil;
    FQuery.Transaction := nil;
  end;
  inherited;
end;

function TsohoIBAssist.ParamExists(const ParamName: string): boolean;
begin
  Result := FQuery.Params.ByName[ParamName] <> nil;
end;

procedure TsohoIBAssist.PrepareQuery;
begin
  FQuery.SQL.Assign(FSQL);
  FQuery.Prepare;
end;

procedure TsohoIBAssist.SetDataSet(Value: TDataSet);
begin
  inherited;
  if Value = nil then begin
    FQuery.Database := nil;
    FQuery.Transaction := nil;
  end
  else begin
    FQuery.Database := (DataSet as TsohoIBDataSet).Database;
    FQuery.Transaction := (DataSet as TsohoIBDataSet).Transaction;
  end;
end;

procedure TsohoIBAssist.SetDateParam(const ParamName: string;
  const Value: TDate);
begin
  FQuery.Params.ByName[ParamName].AsDate := Value;
end;

procedure TsohoIBAssist.SetDateTimeParam(const ParamName: string;
  const Value: TDateTime);
begin
  FQuery.Params.ByName[ParamName].AsDateTime := Value;
end;

procedure TsohoIBAssist.SetFltParam(const ParamName: string;
  const Value: Double);
begin
  FQuery.Params.ByName[ParamName].AsFloat := Value;
end;

procedure TsohoIBAssist.SetIntParam(const ParamName: string;
  const Value: Integer);
begin
  FQuery.Params.ByName[ParamName].AsInteger := Value;
end;

procedure TsohoIBAssist.SetParamNull(const ParamName: string);
begin
  FQuery.Params.ByName[ParamName].IsNull := true;
end;

procedure TsohoIBAssist.SetStrParam(const ParamName, Value: string);
begin
  FQuery.Params.ByName[ParamName].AsString := Value;
end;

procedure TsohoIBAssist.SetTimeParam(const ParamName: string;
  const Value: TTime);
begin
  FQuery.Params.ByName[ParamName].AsTime := Value;
end;

function TsohoIBAssist.TypeRequest(Edit: TsohoCustomEdit): integer;
begin
  case Edit.ValueKind of
    vkString: Result := 0;
    vkInteger: Result := 1;
    vkFloat: Result := 2;
    vkDate: begin
      if FQuery.Database.SQLDialect = 3 then
        Result := 3
      else
        Result := 5
    end;
    vkTime: begin
      if FQuery.Database.SQLDialect = 3 then
        Result := 4
      else
        Result := 5
    end;
    else Result := 5;
  end;
end;

procedure TsohoIBAssist.InsertData;
begin
  if InsertKeyField and  (FGeneratorName <> EmptyStr) then KeyValue := GetNewID(FGeneratorName);
  inherited;
end;


end.